// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.IO;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.Attributes;
using UVa.GCG.WSRF.Service.BaseTypes;
using UVa.GCG.WSRF.Service.ResourceLifetime;

using Grid;
using Grid.ContextSpace;

namespace UVa.GBG.BasicFilesServer.PortTypes
{
	/// <summary>
	/// This is the service code for the BasicFile port type.  A basic
	/// file is a file for which the data is in fact stored on local disk,
	/// but the name on disk and the structure of the directories, etc. have
	/// no bearing on what is visible through context space.  In otherwords,
	/// the back-end disk is merely a convenient storage mechanism and
	/// nothing else.
	/// </summary>
	[WsdlBaseName("BasicFile",
		 "http://cs.virginia.edu/gbg/BasicFiles")]
	[WebServiceBinding]
	[WSRFPortType(typeof(ImmediateResourceTerminationPortType))]
	public class BasicFilePortType : ServiceSkeleton
	{
		#region Resources

		/// <summary>
		/// Just a resource to remember where on disk we put the file.
		/// </summary>
		[Resource]
		private string _fileName;

		#endregion

		#region Resource Properties

		/// <summary>
		/// In the future, any "stat" like properties that we want a file
		/// to have will be considered part of the ResourceProperties
		/// document.  This property gives the size (in bytes) of the file
		/// as it currently stands.
		/// </summary>
		[ResourceProperty(
			 GridConstants.FILE_SIZE_PROPERTY_NAME, 
			 typeof(long), false, false, "1", "1",
			 Namespace=GridConstants.FILE_NS)]
		public long Size
		{
			get
			{
				FileInfo info = new FileInfo(FullPath);
				return info.Length;
			}
		}

		#endregion

		#region Constants
		
		private const string _CANT_CREATE_FILE = "Unable to create scratch storage space.";
		private const string _DEFAULT_SCRATCH_SPACE = ".";
		private const string _SCRATCH_SPACE_PARAM_NAME = "Scratch-Space";
		private const int _MAXIMUM_FILE_CREATION_TRIES = 5;

		#endregion
	
		#region Private Members and Properties

		private string FullPath
		{
			get
			{
				return string.Format("{0}/{1}", ScratchSpace,
					_fileName);
			}
		}

		private string _scratchSpace = null;
		private string ScratchSpace
		{
			get
			{
				if (_scratchSpace == null)
				{
					_scratchSpace = ServiceBase.ServiceParameters[_SCRATCH_SPACE_PARAM_NAME];
					if (_scratchSpace == null)
						_scratchSpace = _DEFAULT_SCRATCH_SPACE;
				}

				return _scratchSpace;
			}
		}

		/// <summary>
		/// While our WSRF resources will be cleaned up automatically by
		/// WSRF, we use this handler so that when we are about to be
		/// deleted we have a chance to clean up the file stored in the back
		/// end file system (which is not technically considered part of
		/// WSRF.NET's managed resources).
		/// </summary>
		/// <param name="resourceID">This parameter gives the resource ID
		/// of the "current" resource, though that information isn't
		/// needed for this metdho.</param>
		private void deletionHandler(string resourceID)
		{
			File.Delete(FullPath);
		}

		#endregion

		#region Web Methods

		/// <summary>
		/// This is the GBG file read method.  All reads are done in
		/// blocks and the absolute position in the file is always given.
		/// </summary>
		/// <param name="offset">The offset into the file at which to
		/// begin reading.</param>
		/// <param name="size">The size of the block of data to read.</param>
		/// <returns>The data read.  Because this is a web method, we are
		/// going to transfer the data on the wire as base64 so this string
		/// is that base64 representation.</returns>
		[WebMethod]
		[SoapDocumentMethod(
			 GridConstants.FILE_READ_ACTION,
			 ParameterStyle=SoapParameterStyle.Bare)]
		[return: XmlElement(GridConstants.FILE_READ_RESULT_ROOT,
			 Namespace=GridConstants.FILE_NS)]
		public string read(
			[XmlElement(GridConstants.FILE_READ_OFFSET_ROOT, 
				 Namespace=GridConstants.FILE_NS)]
				long offset, 
			[XmlElement(GridConstants.FILE_READ_SIZE_ROOT,
				 Namespace=GridConstants.FILE_NS)]
				int size)
		{
			byte []data = new byte[size];
			
			lock(GetType())
			{
				using (FileStream stream = File.Open(FullPath,
						   FileMode.Open, FileAccess.Read, FileShare.None))
				{
					stream.Seek(offset, SeekOrigin.Begin);
					int bytesRead = stream.Read(data, 0, data.Length);
					return Convert.ToBase64String(data, 0, bytesRead);
				}
			}
		}

		/// <summary>
		/// This is the GBG file write method.  All writes are done in blocks
		/// and the absolute position in the file is alwasy given.
		/// </summary>
		/// <param name="offset">The offset into the file at which to
		/// begin writing.</param>
		/// <param name="b64data">As with the read method, the data to write
		/// is transfered as a base64 string.  This is that data.</param>
		/// <returns>The actual number of bytes written.  For this
		/// particular implementation, this will always be the amount
		/// requested unless an exception is thrown, but in general, it may
		/// be useful to allow short writes.</returns>
		[WebMethod]
		[SoapDocumentMethod(GridConstants.FILE_WRITE_ACTION,
			 ParameterStyle=SoapParameterStyle.Bare)]
		[return: XmlElement(GridConstants.FILE_WRITE_RESULT_ROOT,
			 Namespace=GridConstants.FILE_NS)]
		public int write(
			[XmlElement(GridConstants.FILE_WRITE_OFFSET_ROOT,
				Namespace=GridConstants.GRID_NS)]
				long offset,
			[XmlElement(GridConstants.FILE_WRITE_DATA_ROOT, 
				Namespace=GridConstants.FILE_NS)]
				string b64data)
		{
			byte[] data = Convert.FromBase64String(b64data);
			
			lock(GetType())
			{
				using (FileStream stream = File.Open(FullPath,
						   FileMode.Open, FileAccess.Write, FileShare.None))
				{
					stream.Seek(offset, SeekOrigin.Begin);
					stream.Write(data, 0, data.Length);

					return data.Length;
				}
			}
		}

		/// <summary>
		/// This is the truncAppend method of the file port type.  Usually
		/// you would imagine doing a truncate, and then writing to the
		/// file, but it makes since in a WAN environment to combine the
		/// two operations for efficiency.
		/// </summary>
		/// <param name="b64data">As with read and write, the data that is
		/// transfered to the file to write is passed as a base64 string.
		/// This parameter is that data.</param>
		/// <returns>The actual number of bytes written.  For this
		/// particular implementation, this will always be the amount
		/// requested unless an exception is thrown, but in general, it may
		/// be useful to allow short writes.</returns>
		[WebMethod]
		[SoapDocumentMethod(GridConstants.FILE_TRUNCAPPEND_ACTION,
			ParameterStyle=SoapParameterStyle.Bare)]
		[return: XmlElement(GridConstants.FILE_TRUNCAPPEND_RESULT_ROOT,
			Namespace=GridConstants.FILE_NS)]
		public int truncAppend(
			[XmlElement(GridConstants.FILE_TRUNCAPPEND_DATA_ROOT,
				Namespace=GridConstants.FILE_NS)]
			string b64data)
		{
			byte[] data = Convert.FromBase64String(b64data);
			
			lock(GetType())
			{
				using (FileStream stream = File.Open(FullPath,
						   FileMode.Truncate, FileAccess.Write, FileShare.None))
				{
					stream.Write(data, 0, data.Length);

					return data.Length;
				}
			}
		}

		/// <summary>
		/// This is the append method of the file port type.
		/// </summary>
		/// <param name="b64data">As with read and write, the data that is
		/// transfered to the file to write is passed as a base64 string.
		/// This parameter is that data.</param>
		/// <returns>The actual number of bytes written.  For this
		/// particular implementation, this will always be the amount
		/// requested unless an exception is thrown, but in general, it may
		/// be useful to allow short writes.</returns>
		[WebMethod]
		[SoapDocumentMethod(GridConstants.FILE_APPEND_ACTION,
			ParameterStyle=SoapParameterStyle.Bare)]
		[return: XmlElement(GridConstants.FILE_APPEND_RESULT_ROOT,
			Namespace=GridConstants.FILE_NS)]
		public int append(
			[XmlElement(GridConstants.FILE_APPEND_DATA_ROOT,
				 Namespace=GridConstants.FILE_NS)]
			string b64data)
		{
			byte[] data = Convert.FromBase64String(b64data);
			
			lock(GetType())
			{
				using (FileStream stream = File.Open(FullPath, FileMode.Append,
						   FileAccess.Write, FileShare.None))
				{
					stream.Write(data, 0, data.Length);

					return data.Length;
				}
			}
		}

		#endregion

		#region WSRF.NET Methods

		/// <summary>
		/// The portInit method is called by the WSRF.NET system every single
		/// time the web service class is instantiated.  It essentially acts
		/// like a constructor except that it is guaranteed to be called
		/// only after all of the information about the base class has been
		/// set up properly.
		/// </summary>
		protected override void portInit()
		{
			ServiceBase.addDeletionListener(
				new WebServiceBase.DeletionDelegate(deletionHandler));
		}

		/// <summary>
		/// The InitResource method is called by the WSRF.NET system every
		/// time a new endpoint is created.  This acts like an EPR
		/// constructor and gives us an opportunity to initialize first time
		/// data for our service endpoint.
		/// </summary>
		/// <param name="parameters"></param>
		public override void InitResource(Hashtable parameters)
		{
			FileStream stream = null;
			int lcv;
		
			// It turns out that there isn't a convenient way to ask the
			// windows operating system to give us a brand new, completely
			// unique file name for a given directory.  We could look through
			// the directory and see what's there, but it's just as easy to
			// simple try and create the file a couple of times and see what
			// happens.  We loop some small number of times in the very
			// unlikely case that we guess wrong originally, or in case of
			// race conditions.
			for (lcv = 0; lcv < _MAXIMUM_FILE_CREATION_TRIES; lcv++)
			{
				// Generate a new, random file name.
				_fileName = Guid.NewGuid().ToString();
				try
				{
					// We'll attempt to create a new file with read and
					// write access (which are the permissions we'll need
					// any ways).  If this fails, it will through an
					// IOException which we catch.
					stream = File.Open(FullPath,
						FileMode.CreateNew, FileAccess.ReadWrite);
					return;
				}
				catch (IOException)
				{
					// Couldn't create that file, let's try again.
					continue;
				}
				finally
				{
					if (stream != null)
						stream.Close();
				}
			}

			// If we got this far, then we couldn't create the file in the
			// requisite number of trys so throw up our hands and give up.
			if (lcv >= _MAXIMUM_FILE_CREATION_TRIES)
				throw new ServiceException(_CANT_CREATE_FILE);
		}

		#endregion
	}
}