// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;

using Grid;
using Grid.Commands;
using Grid.Env;

namespace GridCommand
{
	/// <summary>
	/// This class is the "MAIN" class for the grid command tool.  There
	/// isn't a whole lot of code in here because the majority of the work
	/// is actually performed by tools in the grid library rather then in
	/// this command line code.  This will make it easier to re-use that
	/// functionallity later in other tools, etc.
	/// </summary>
	class GridCommand
	{
		/// <summary>
		/// The Main method that is called when the tool is run.
		/// </summary>
		/// <param name="args">Command line arguments.</param>
		[STAThread]
		static void Main(string[] args)
		{
			bool debug = false;

			try
			{
				// We parse out the information (which might include
				// tool options) up to and including the name of the command
				// to run.  We also then take the remaining args and package
				// them up nice and neat for the command class itself to
				// use.
				GridParseData gpd = Parsing.parse(args);
				debug = gpd.Debug;

				// Set the appropriate debug and verbosity streams as
				// necessary.
				if (gpd.Debug)
					GridLibEnvironment.Instance.Console.Debug = Console.Out;
				if (gpd.Verbose)
					GridLibEnvironment.Instance.Console.Trace = Console.Out;

				// First, given the name of the command, look for a factory
				// that knows how to create those commands.
				ICommandFactory factory = CommandCollection.Instance[
					gpd.CommandName];
				// Then, assuming we found a factory, create the command and
				// run it.
				ICommand cmd = factory.create(gpd.CommandArgs);
				cmd.run();
			}
			catch (GridException ge)
			{
				if (debug)
					Console.Error.WriteLine(ge);
				else
					Console.Error.WriteLine(ge.Message);

				Environment.Exit(1);
			}
		}
	}
}