// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;

using Grid;
using Grid.Extensions;

namespace Grid.Commands
{
	/// <summary>
	/// Represents a collection of all of the commands currently known by
	/// the command environment.  This class is a singleton class and therefor
	/// only one instance per AppDomain will be created.
	/// </summary>
	public class CommandCollection
	{
		static private CommandCollection _instance = new CommandCollection();

		/// <summary>
		/// Retrieve the singleton instance.
		/// </summary>
		static public CommandCollection Instance
		{
			get
			{
				return _instance;
			}
		}

		/// <summary>
		/// We store the known factories in a hash-table to make access to
		/// them relatively quick.  The hashtable stores the following
		/// mapping:
		/// <code>[command-name] -> factory-instance </code>
		/// </summary>
		private Hashtable _factories = new Hashtable();

		/// <summary>
		/// We make the constructor for this class private since it is
		/// intended to be a singleton and we don't really want other people
		/// creating new instances of them.
		/// </summary>
		private CommandCollection()
		{
			add(new DefaultCommandFactory(typeof(GridUsageCommand)));
			add(new DefaultCommandFactory(typeof(GridLibConfCommand)));
			add(new DefaultCommandFactory(typeof(GridInitCommand)));
			add(new DefaultCommandFactory(typeof(GridConnectCommand)));
			add(new DefaultCommandFactory(typeof(GridDisconnectCommand)));
			add(new DefaultCommandFactory(typeof(GridLsCommand)));
			add(new DefaultCommandFactory(typeof(GridMkdirCommand)));
			add(new DefaultCommandFactory(typeof(GridLnCommand)));
			add(new DefaultCommandFactory(typeof(GridCpCommand)));
			add(new DefaultCommandFactory(typeof(GridCdCommand)));
			add(new DefaultCommandFactory(typeof(GridRmCommand)));
			add(new DefaultCommandFactory(typeof(GridInfoCommand)));
			add(new DefaultCommandFactory(typeof(GridPropertiesCommand)));

			addExtensions();
		}

		private void addExtensions()
		{
			foreach (ExtensionLibraryDescription desc in 
				ExtensionCollection.Instance.InstalledExtensionLibraries)
			{
				foreach (ICommandFactory factory in desc.CommandFactories)
				{
					add(factory);
				}
			}
		}

		/// <summary>
		/// Add a new command factory instance to the catalog.
		/// </summary>
		/// <param name="cmd">The new command factory to add.</param>
		private void add(ICommandFactory cmd)
		{
			_factories.Add(cmd.CommandName, cmd);
		}

		/// <summary>
		/// Find a command factory for the given command name.  This property
		/// throws an UnknownCommandException exception if the indicated
		/// command factory canoot be found.
		/// </summary>
		public ICommandFactory this[string commandName]
		{
			get
			{
				ICommandFactory cmd = _factories[commandName] as ICommandFactory;
				if (cmd == null)
					throw new UnknownCommandException(commandName);

				return cmd;
			}
		}

		/// <summary>
		/// Retrieve all of the known command factories.
		/// </summary>
		public ICommandFactory[] Commands
		{
			get
			{
				ICommandFactory []ret = new
					ICommandFactory[_factories.Values.Count];
				_factories.Values.CopyTo(ret, 0);

				return ret;
			}
		}
	}
}