// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Reflection;

namespace Grid.Commands
{
	/// <summary>
	/// All commands have an associated command factory that "knows" hot to
	/// create them.  However, using reflection it is possible to write a
	/// single factory that can be "told" how to create each individual
	/// command separately.  This class is such a factory.
	/// </summary>
	public class DefaultCommandFactory : AbstractCommandFactory
	{
		private const string _PROPERTY_NAME = "CommandName";
		private const string _PROPERTY_USAGE = "Usage";

		private Type _commandClass;

		/// <summary>
		/// Create a new DefaultCommandFactory instance.
		/// </summary>
		/// <param name="commandClass">The Type or class of the command
		/// that this factory will be responsible for creating.  It is
		/// required that whatever type is passed in to this constructor
		/// must implement the ICommand interface and it must have a 
		/// constructor which takes as it's single argument an
		/// ICommandLineParser parameter.</param>
		public DefaultCommandFactory(Type commandClass)
			: base(getPropertyValue(commandClass, _PROPERTY_NAME), 
				getPropertyValue(commandClass, _PROPERTY_USAGE))
		{
			_commandClass = commandClass;

			// Make sure that the command class given to us in fact
			// implements the required interface (ICommand).
			if (!((typeof(ICommand)).IsAssignableFrom(_commandClass)))
				throw new CommandException(string.Format(
					"\"{0}\" does not implement ICommand interface.", 
					_commandClass.FullName));
		}

		/// <summary>
		/// This "business" method is the one used when an instance of
		/// a command needs to be created.
		/// </summary>
		/// <param name="commandLine">The commandline arguments that are
		/// to be given to the new command so that it can run.</param>
		/// <returns>A new ICommand instance of the appropriate type.</returns>
		public override ICommand create(ICommandLineParser commandLine)
		{
			try
			{
				ConstructorInfo info = _commandClass.GetConstructor(
					new Type[] {typeof(ICommandLineParser)});

				// If we didn't find the constructor, it's because there wasn't
				// one with the correct parameter list.
				if (info == null)
					throw new CommandException(string.Format(
						"Type \"{0}\" does not contain a public constructor that "
						+ "takes a ICommandLineParser instance as a parameter.",
						_commandClass.FullName));

				return (ICommand)info.Invoke(new object[] {commandLine});
			}
			catch (GridException)
			{
				throw;
			}
			catch (TargetInvocationException tie)
			{
				if (tie.InnerException is GridException)
					throw (GridException)tie.InnerException;

				throw;
			}
			catch (Exception e)
			{
				throw new UnexpectedException(e);
			}
		}

		#region Statics

		static private string getPropertyValue(Type commandClass, string propertyName)
		{
			try
			{
				PropertyInfo info = commandClass.GetProperty(propertyName,
					BindingFlags.Static | BindingFlags.Public, null, typeof(string),
					new Type[0], null);
				if (info == null)
					throw new CommandException(
						string.Format("Type \"{0}\" does not contain a \"{1}\" static property.",
						commandClass.FullName, propertyName));
			
				MethodInfo method = info.GetGetMethod();
				if (method == null)
					throw new CommandException(
						string.Format("Type \"{0}\" does not contain a \"{1}\" static "
						+ "property with a get accessor.",
						commandClass.FullName, propertyName));
				return (string)method.Invoke(null, null);
			}
			catch (CommandException)
			{
				throw;
			}
			catch (Exception e)
			{
				throw new UnexpectedException(e);
			}
		}

		#endregion
	}
}