// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;

namespace Grid.Commands
{
	/// <summary>
	/// This is the default version of the command line parser interface.
	/// This class parses command lines very simply by assuming that any
	/// string that starts with a double-dash "--" is a long command line
	/// option, any that starts with a single-dash "-" is a short command
	/// line option, and everything else is a command line argument.
	/// Further, it assumes that all short options are single letters and
	/// can be grouped together.  Also, all options are assumed to have
	/// optional values associated with them.  These options are combined
	/// together as the parser runs, and can also be listed together and
	/// separated by commas.  For example:
	/// 
	/// <code>
	///		command -short=a,b,c -t=d
	/// </code>
	/// 
	/// The above code example would create a commandline parser with 5,
	/// short options called s, h, o, r, and t.  t would also have 4
	/// values associated with it -- a, b, c and d.
	/// </summary>
	public class DefaultCommandLineParser : ICommandLineParser
	{
		private string []_arguments;

		/// <summary>
		/// Maps option name to ArrayList of option values.
		/// </summary>
		private Hashtable _options = new Hashtable();

		/// <summary>
		/// Create a new DefaultCommandLineParser with the given command
		/// line.
		/// </summary>
		/// <param name="commandLine">The commandline to parse.</param>
		/// <param name="startIndex">The index in the commandline array at
		/// which to start parsing.</param>
		public DefaultCommandLineParser(string []commandLine,
			int startIndex)
		{
			ArrayList arguments = new ArrayList();

			for (int lcv = startIndex; lcv < commandLine.Length; lcv++)
			{
				// This parsing isn't the most clever in the world.  In
				// particular, one could implement something slighly more
				// complex that would allow the user to specify a change of
				// option indicator such as:
				//
				//   --long-prefix=++
				//
				// But this is left for a later date.
				if (commandLine[lcv].StartsWith("--"))
					parseLongOption(commandLine[lcv]);
				else if (commandLine[lcv].StartsWith("-"))
					parseShortOption(commandLine[lcv]);
				else
					arguments.Add(commandLine[lcv]);
			}

			_arguments = new string[arguments.Count];
			arguments.CopyTo(_arguments);
		}

		#region Private Members

		private void parseLongOption(string arg)
		{
			arg = arg.Substring(2);
			int index = arg.IndexOf('=');
			if (index > 0)
			{
				addOption(arg.Substring(0, index), 
					arg.Substring(index + 1));
			} 
			else
			{
				addOption(arg, null);
			}
		}

		private void parseShortOption(string arg)
		{
			arg = arg.Substring(1);
			int index = arg.IndexOf('=');
			if (index <= 0)
			{
				for (int lcv = 0; lcv < arg.Length; lcv++)
					addOption(new string(arg[lcv], 1), null);
			} 
			else
			{
				int lcv;
				for (lcv = 0; lcv < index - 1; lcv++)
					addOption(new string(arg[lcv], 1), null);
				addOption(new string(arg[lcv], 1), arg.Substring(index + 1));
			}
		}

		private void addOption(string optName, string val)
		{
			ArrayList list = _options[optName] as ArrayList;
			if (list == null)
				_options[optName] = list = new ArrayList();
			if (val != null)
			{
				string []vals = val.Split(',');
				list.AddRange(vals);
			}
		}

		#endregion

		#region ICommandLineParser Members

		/// <summary>
		/// <see cref="Grid.Commands.ICommandLineParser.Arguments"/>
		/// </summary>
		public string[] Arguments
		{
			get
			{
				return _arguments;
			}
		}

		/// <summary>
		/// <see cref="Grid.Commands.ICommandLineParser.hasOption"/>
		/// </summary>
		public bool hasOption(string optionName)
		{
			return _options.ContainsKey(optionName);
		}

		/// <summary>
		/// <see cref="Grid.Commands.ICommandLineParser.this[string]"/>
		/// </summary>
		public string this[string optionName]
		{
			get
			{
				ArrayList list = _options[optionName] as ArrayList;
				if (list == null)
					return null;
				if (list.Count == 0)
					return string.Empty;
				return list[0] as string;
			}
		}

		/// <summary>
		/// <see cref="Grid.Commands.ICommandLineParser.allOptionValues"/>
		/// </summary>
		public string[] allOptionValues(string optionName)
		{
			string []vals;
			ArrayList list = _options[optionName] as ArrayList;
			if (list == null)
				return null;
			vals = new string[list.Count];
			list.CopyTo(vals);
			return vals;
		}

		/// <summary>
		/// <see cref="Grid.Commands.ICommandLineParser.Options"/>
		/// </summary>
		public string[] Options
		{
			get
			{
				string []ret;
				ret = new string[_options.Keys.Count];
				_options.Keys.CopyTo(ret, 0);
				return ret;
			}
		}

		#endregion
	}
}