// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

using Grid.Operations;

namespace Grid.Commands
{
	/// <summary>
	/// This command is used when the user wishes to create a brand new grid
	/// somewhere.  This class will create the root context for that grid
	/// as well as the Factories context and will also add factories into the
	/// Factories context.  Users wishing to do this from code should use the
	/// GridInitOperation rather then the GridInitCommand.
	/// 
	/// NOTE:  Please see <see cref="Grid.Operations.GridInitOperation"/> for
	/// important information about using GridInit.
	/// </summary>
	public class GridInitCommand : ICommand
	{
		#region Default Factory Tokens

		/// <summary>
		/// The name of this command.
		/// </summary>
		static public string CommandName
		{
			get
			{
				return "init";
			}
		}

		/// <summary>
		/// The usage string which describes this command.
		/// </summary>
		static public string Usage
		{
			get
			{
				return "init <config-file> <output-file>\n"
					+ "\tOR\n"
					+ "init --example-config";
			}
		}

		#endregion

		private bool _exampleConfig = false;
		private GridInitConfig _gridConfig = null;
		private string _gridFile = null;

		/// <summary>
		/// Create a new GridInitCommand with the given command line.
		/// </summary>
		/// <param name="cmdLine">The CommandLine arguments to create
		/// this command with.</param>
		public GridInitCommand(ICommandLineParser cmdLine)
		{
			if (cmdLine.Options != null && cmdLine.Options.Length > 0)
			{
				if (cmdLine.hasOption("example-config"))
				{
					_exampleConfig = true;
					return;
				}
				else
					throw new CommandUsageException(CommandName, Usage);
			}

			string []args = cmdLine.Arguments;
			if (args == null || args.Length != 2)
				throw new CommandUsageException(CommandName, Usage);

			_gridConfig = parseConfig(args[0]);
			_gridFile = args[1];
		}

		#region ICommand Members

		/// <summary>
		/// <see cref="Grid.Commands.ICommand.run"/>
		/// </summary>
		public void run()
		{
			if (_exampleConfig)
				GridInitOperation.showExampleConfig();
			else
				GridInitOperation.init(_gridConfig, _gridFile);
		}

		#endregion

		#region Private Members

		static private GridInitConfig parseConfig(string file)
		{
			using (StreamReader reader = new StreamReader(file))
			{
				XmlSerializer ser = new XmlSerializer(typeof(GridInitConfig));
				return (GridInitConfig)ser.Deserialize(reader);
			}
		}

		#endregion
	}
}