// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.WS;
using UVa.GCG.WSRF.Common.WS.Addressing;

using Grid;

namespace Grid.Commands
{
	/// <summary>
	/// Stores and gives access to configuration information for a
	/// single factory to be created in the grid.  This instance can
	/// be created programmatically, but is also intended to be parsed
	/// directly out of an XML Configuration file.
	/// </summary>
	[XmlRoot(GridConstants.GRID_INIT_FACTORY_ROOT,
		 Namespace=GridConstants.GRID_NS)]
	public class GridFactoryInfo
	{
		private string _type;
		private string _name;
		private EndpointReferenceType _factory;

		/// <summary>
		/// Create a new GridFactoryInfo type with given parameters.
		/// </summary>
		/// <param name="type">The grid "type" which this factory will
		/// create when invoked.</param>
		/// <param name="name">The name that the factory is to be
		/// given in context space.</param>
		/// <param name="factory">The EPR of the factory itself.</param>
		public GridFactoryInfo(
			XmlQualifiedName type,
			string name,
			EndpointReferenceType factory)
		{
			_type = type.ToString();
			_name = name;
			_factory = factory;
		}

		/// <summary>
		/// This default constructor is here only because it is required
		/// as part of the Xml Serialization mechanism.
		/// </summary>
		public GridFactoryInfo()
		{
		}

		/// <summary>
		/// Retrieve or set the type which this factory will create.
		/// </summary>
		[XmlElement(GridConstants.GRID_FACTORY_TYPE_ELEMENT,
			 Namespace=GridConstants.GRID_NS)]
		public string FactoryType
		{
			get
			{
				return _type;
			}

			set
			{
				_type = value;
			}
		}

		/// <summary>
		/// Retrieve or set the name of this factory in context space.
		/// </summary>
		[XmlAttribute(GridConstants.GRID_FACTORY_NAME_ATTR,
			 Namespace=GridConstants.GRID_NS)]
		public string FactoryName
		{
			get
			{
				return _name;
			}

			set
			{
				_name = value;
			}
		}

		/// <summary>
		/// Retrieve or set the EPR of the factory instance.
		/// </summary>
		[XmlElement(GridConstants.GRID_FACTORY_ENDPOINT_ELEMENT,
			 Namespace=GridConstants.GRID_NS)]
		public EndpointReferenceType FactoryEPR
		{
			get
			{
				return _factory;
			}

			set
			{
				_factory = value;
			}
		}
	}

	/// <summary>
	/// Stores and gives access to configuration information for the
	/// initialization of a new grid.  This file can be created
	/// programatically as needed, but is also intended to be parsed
	/// directly from an XML configuration file (hence, the XmlSerialization
	/// tags).
	/// </summary>
	[XmlRoot(GridConstants.GRID_INIT_CONFIG_ROOT,
		 Namespace=GridConstants.GRID_NS)]
	public class GridInitConfig
	{
		private GridFactoryInfo []_infos;

		/// <summary>
		/// The default constructor for this type.
		/// </summary>
		public GridInitConfig()
		{
		}

		/// <summary>
		/// Retrieve or set the array of factories that this configuration
		/// instance contains.
		/// </summary>
		[XmlArray(GridConstants.GRID_FACTORIES_ARRAY_ELEMENT,
			 Namespace=GridConstants.GRID_NS)]
		[XmlArrayItem(GridConstants.GRID_FACTORY_ARRAY_ITEM_ELEMENT,
			 Namespace=GridConstants.GRID_NS)]
		public GridFactoryInfo[] Factories
		{
			get
			{
				return _infos;
			}

			set
			{
				_infos = value;
			}
		}
	}
}