// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;

using Grid.Operations;
using Grid.Env;
using Grid.Extensions;

namespace Grid.Commands
{
	/// <summary>
	/// This command is used when the user wishes to acquire or modify
	/// information about the current grid lib environment.
	/// </summary>
	public class GridLibConfCommand : ICommand
	{
		#region Constants

		private const string _INSTALL_OPTION = "install-ext-lib";
		private const string _REMOVE_OPTION = "remove-ext-lib";
		private const string _LIST_OPTION = "list-ext-libs";
		private const string _NAME_OPTION = "name";
		private const string _DESCRIPTION_OPTION = "description";

		#endregion

		#region Default Factory Tokens

		/// <summary>
		/// The name of this command.
		/// </summary>
		static public string CommandName
		{
			get
			{
				return "lib-conf";
			}
		}

		/// <summary>
		/// The usage string which describes this command.
		/// </summary>
		static public string Usage
		{
			get
			{
				return "lib-conf --install-ext-lib [--name=<name>] [--description=<desc>] <assembly-file>\n"
					+ "\tOR\n"
					+ "lib-conf --remove-ext-lib <name>\n"
					+ "\tOR\n"
					+ "lib-conf --list-ext-libs";
			}
		}

		#endregion

		private string _name = null;
		private string _description = null;
		private string _file = null;

		/// <summary>
		/// Create a new GridLibConfCommand with the given command line.
		/// </summary>
		/// <param name="cmdLine">The CommandLine arguments to create
		/// this command with.</param>
		public GridLibConfCommand(ICommandLineParser cmdLine)
		{
			if (cmdLine.hasOption(_INSTALL_OPTION))
			{
				if (cmdLine.Arguments.Length != 1)
					throw new CommandUsageException(CommandName, Usage);
				_file = cmdLine.Arguments[0];

				_name = cmdLine[_NAME_OPTION];
				_description = cmdLine[_DESCRIPTION_OPTION];
			} 
			else if (cmdLine.hasOption(_REMOVE_OPTION))
			{
				if (cmdLine.Arguments.Length != 1)
					throw new CommandUsageException(CommandName, Usage);
				_name = cmdLine.Arguments[0];
			} 
			else if (cmdLine.hasOption(_LIST_OPTION))
			{
				if (cmdLine.Arguments.Length != 0)
					throw new CommandUsageException(CommandName, Usage);
			} else
				throw new CommandUsageException(CommandName, Usage);
		}

		#region ICommand Members

		/// <summary>
		/// <see cref="Grid.Commands.ICommand.run"/>
		/// </summary>
		public void run()
		{
			if (_file != null)
				GridLibConfOperation.addExtensionLibrary(
					_file, _name, _description);
			else if (_name != null)
			{
				GridLibConfOperation.removeExtensionLibrary(
					_name);
			} 
			else
			{
				ExtensionLibraryDescription[] descriptions =
					GridLibConfOperation.InstalledExtensionLibraries;
				foreach (ExtensionLibraryDescription desc in descriptions)
				{
					GridLibEnvironment.Instance.Console.Output.WriteLine(
						"{0}", desc);
				}
			}
		}

		#endregion
	}
}