// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Xml;

using Grid.Operations;
using Grid.ContextSpace;

namespace Grid.Commands
{
	/// <summary>
	/// This command is used when the user wishes to link an EPR into a spot in
	/// the grid.  Users wishing to do this from code should use the 
	/// GridLnOperation rather then the GridLnCommand.
	/// </summary>
	public class GridLnCommand : ICommand
	{
		#region Default Factory Tokens

		/// <summary>
		/// The name of this command.
		/// </summary>
		static public string CommandName
		{
			get
			{
				return "ln";
			}
		}

		/// <summary>
		/// The usage string which describes this command.
		/// </summary>
		static public string Usage
		{
			get
			{
				return
					"ln [ --context | --file | --factory | --qname=<type-qname> ] "
					+ "<source-grid-endpoint> [<target-grid-path>]\n"
					+ "\tWHERE\n"
					+ "<grid-endpoint> is:\n"
					+ "\t<grid-path>\n"
					+ "\t\tOR\n"
					+ "\tepr-file:<grid-path>";
			}
		}

		#endregion

		private XmlQualifiedName _suggestedType = null;
		private string _sourceEndpointDescription = null;
		private PathExpressionCollection _target = null;

		/// <summary>
		/// Create a new GridCdCommand with the given command line.
		/// </summary>
		/// <param name="cmdLine">The CommandLine arguments to create
		/// this command with.</param>
		public GridLnCommand(ICommandLineParser cmdLine)
		{
			// Right now we only understand three types of entries.  In
			// the future, it makes sense to make this more generic, but
			// for now, this will do.
			if (cmdLine.Options != null && cmdLine.Options.Length > 0)
			{
				if (cmdLine.hasOption("context"))
					_suggestedType = WellknownEntryTypes.ENTRY_TYPE_CONTEXT;
				else if (cmdLine.hasOption("file"))
					_suggestedType = WellknownEntryTypes.ENTRY_TYPE_FILE;
				else if (cmdLine.hasOption("factory"))
					_suggestedType = WellknownEntryTypes.ENTRY_TYPE_FACTORY;
				else if (cmdLine.hasOption("qname"))
				{
					string qname = cmdLine["qname"];
					if (qname == null)
						_suggestedType = WellknownEntryTypes.ENTRY_TYPE_UNKNOWN;
					else
					{
						int index = qname.LastIndexOf(':');
						if (index > 0)
							_suggestedType = new XmlQualifiedName(
								qname.Substring(index + 1), qname.Substring(0, index));
						else
							_suggestedType = WellknownEntryTypes.ENTRY_TYPE_UNKNOWN;
					}
				} else
					_suggestedType = WellknownEntryTypes.ENTRY_TYPE_UNKNOWN;
			}

			string []args = cmdLine.Arguments;
			if (args == null || args.Length == 0 || args.Length > 2)
				throw new CommandUsageException(CommandName, Usage);

			_sourceEndpointDescription = args[0];
			if (args.Length == 2)
				_target = new PathExpressionCollection(args[1]);
		}

		#region ICommand Members

		/// <summary>
		/// <see cref="Grid.Commands.ICommand.run"/>
		/// </summary>
		public void run()
		{
			GridLnOperation.ln(_sourceEndpointDescription, _target,
				_suggestedType);
		}

		#endregion
	}
}