// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;

namespace Grid.Commands
{
	/// <summary>
	/// This is a common interface which all command line parser should
	/// implement.  This interface is perhaps of questionable useful-ness
	/// as it seems a little unlikely that there will ever be more then
	/// one actual instance of the command line parser, and even if there
	/// were, it's also questionable whether or not a common base is
	/// needed, but one case that does pop into my mind that might be
	/// useful is where we have a "shell" command which then repeatedly
	/// parses arguments in a different way and hands them off to the
	/// commands that we have already created.  In the end, it costs
	/// nothing to do this and may benefit us later so I did it this
	/// way.
	/// </summary>
	public interface ICommandLineParser
	{
		/// <summary>
		/// Retrieve the list of command line arguments that this
		/// commandline parser has acquired.
		/// </summary>
		string[] Arguments { get; }

		/// <summary>
		/// Determine whether or not a given option exists in the
		/// parsed command line.
		/// </summary>
		/// <param name="optionName">The string name of the option to
		/// check for.</param>
		/// <returns>Whether or not the given option name exists.</returns>
		bool hasOption(string optionName);

		/// <summary>
		/// Retrieve the singleton option value for the given option
		/// name.  This property will return the first value if available
		/// and there are more then one values there.  It will return the
		/// empty string if there are no values available at all, and it
		/// will return null if the option doesn't even exist in the parsed
		/// command line.
		/// </summary>
		string this[string optionName] { get; }

		/// <summary>
		/// Retrieve all of the option values available for a given option
		/// name.
		/// </summary>
		/// <param name="optionName">The name of the option to retrieve
		/// values for.</param>
		/// <returns>An array of string values for a given option name.  This
		/// will be the empty list if there were no values, or null if the
		/// option doesn't even exist.</returns>
		string[] allOptionValues(string optionName);

		/// <summary>
		/// Retrieve the names of all options currently known about by the
		/// parser.
		/// </summary>
		string[] Options { get; }
	}
}
