// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Xml;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An abstract base class which implements much of the required
	/// functionallity from the IEntry interface.  This class is
	/// available to users who wish to gain the functionallity provided
	/// and implement the IEntry interface.  However, it's always possible
	/// for a user to implement this functionallity on their own, thereby
	/// possibly allowing them to derive off of some other class that they
	/// feel is appropriate.
	/// </summary>
	public abstract class AbstractEntry : IEntry
	{
		private XmlQualifiedName _entryType;
		private string _entryName;
		private IEntry _parent;

		/// <summary>
		/// Create a new AbstractEntry type.
		/// </summary>
		/// <param name="entryName">The name that this entry has in
		/// context space (if available).</param>
		/// <param name="entryType">The type of this entry in context
		/// space.  This is assumed to be one of the types listed in
		/// <see cref="Grid.WellknownEntryTypes"/> though this is not
		/// in fact a requirement.</param>
		/// <param name="parent">The parent entry for this entry if
		/// available.  Parent is defined as the context or directory
		/// that contains this entry.  Note however that this is not
		/// strictly guaranteed to be the true parent.  In fact, this
		/// is more or less arbitrary and is merely used in some special
		/// cases when a parent needs to be created.  This parameter can
		/// be left null most of the time.</param>
		protected AbstractEntry(
			string entryName,
			XmlQualifiedName entryType,
			IEntry parent)
		{
			_entryType = entryType;
			_entryName = entryName;
			_parent = parent;
		}

		#region IEntry Members

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.EntryType"/>
		/// </summary>
		public XmlQualifiedName EntryType
		{
			get
			{
				return _entryType;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.EntryName"/>
		/// </summary>
		public string EntryName
		{
			get
			{
				return _entryName;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Parent"/>
		/// </summary>
		public IEntry Parent
		{
			get
			{
				return _parent;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Exists"/>
		/// </summary>
		public bool Exists
		{
			get
			{
				return _entryType != null;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Path"/>
		/// </summary>
		public string Path
		{
			get
			{
				if (Parent == null)
					return EntryName;

				string path = Parent.Path;
				if (path.Equals("/"))
					return path + EntryName;
				
				return path + "/" + EntryName;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Local"/>
		/// </summary>
		public abstract bool Local { get; }

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.format"/>
		/// </summary>
		public abstract LsFormat format(bool isLong);

		#endregion
	}
}