// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An abstract base class for LocalEntries.  See ILocalEntry for details
	/// on what this interface is for.
	/// </summary>
	public abstract class AbstractLocalEntry : IEntry
	{
		/// <summary>
		/// The parent of this IEntry
		/// </summary>
		protected IEntry _parent;

		/// <summary>
		/// The FilesystemInfo which describes this file or directory.
		/// </summary>
		protected FileSystemInfo _info;

		/// <summary>
		/// Create a new AbstractLocalEntry.
		/// </summary>
		/// <param name="parent">The parent, if available, of this entry.
		/// See <see cref="Grid.ContextSpace.IEntry.Parent"/> for details.
		/// </param>
		/// <param name="info">The file system info which describes this
		/// local entry.</param>
		protected AbstractLocalEntry(IEntry parent, FileSystemInfo info)
		{
			_parent = parent;
			_info = info;
		}

		/// <summary>
		/// Indicates whether or not this local entry refers to a directory.
		/// </summary>
		public bool IsDirectory
		{
			get
			{
				return (_info.Attributes & FileAttributes.Directory) > 0;
			}
		}

		#region IEntry Members

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.EntryType"/>
		/// </summary>
		public XmlQualifiedName EntryType
		{
			get
			{
				if (IsDirectory)
					return WellknownEntryTypes.ENTRY_TYPE_CONTEXT;
				else
					return WellknownEntryTypes.ENTRY_TYPE_FILE;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.EntryName"/>
		/// </summary>
		public string EntryName
		{
			get
			{
				return _info.Name;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Exists"/>
		/// </summary>
		public bool Exists
		{
			get
			{
				return _info.Exists;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Parent"/>
		/// </summary>
		public IEntry Parent
		{
			get
			{
				return _parent;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Path"/>
		/// </summary>
		public string Path
		{
			get
			{
				return _info.FullName;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.Local"/>
		/// </summary>
		public bool Local
		{
			get
			{
				return true;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.format"/>
		/// </summary>
		public abstract LsFormat format(bool isLong);

		#endregion
	}
}