// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;

namespace Grid.ContextSpace
{
	/// <summary>
	/// Possible flags that manipulate how context path look-ups are
	/// performed.
	/// </summary>
	[Flags]
	public enum PathLookupFlags
	{
		/// <summary>
		/// Indicates taht the path given must exist.  If this option isn't
		/// used, then path's that do not exist will return IEntry's whose
		/// implemented interfaces include the
		/// <see cref="INotAvailableEntry"/> interface.
		/// </summary>
		MustExist,

		/// <summary>
		/// This flags indicates that only paths which resolve to grid entries
		/// should be resolved.  All others will be filtered out.  This flag is
		/// mutually exclusive with the Local flag.
		/// </summary>
		Grid,

		/// <summary>
		/// This flag indicates that only paths resolving to local entries should
		/// be resolved.  All others will be filtered out.  This flag is
		/// mutually exclusive with the Grid flag.
		/// </summary>
		Local
	}

	/// <summary>
	/// This class contains a few useful utility functions for dealing with
	/// context space paths.  In particular, it contains methods for looking up
	/// paths and retrieving IEntry instances which describe them.  A lot of 
	/// this functionallity is also given in the 
	/// <see cref="Grid.ContextSpace.PathExpressionCollection"/> class (and
	/// in fact, ContextUtilities currently uses that class).  However, in the
	/// future, this functionallity will be split out as PathExpressionCollection
	/// is somewhat unwieldy to use.
	/// </summary>
	public class ContextUtilities
	{
		/// <summary>
		/// Lookup a single entry in context space (or local space).  If the
		/// srting expression given resolves to more then one entry, then an
		/// exception will be thrown indicating that the expression does not
		/// resolve to a singleton.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <param name="flags">Flags which indicate what kind of filtering
		/// should be done.</param>
		/// <param name="requiredTypes">A list of QNames which inidicate which
		/// Context space entry types are permitted.  If this parameter is null,
		/// then all types are permitted.</param>
		/// <returns>The IEntry (if any) which maches the given query.  If no
		/// entry matches, then this result will be null.</returns>
		static public IEntry PathLookup(string pathExpression,
			PathLookupFlags flags, XmlQualifiedName []requiredTypes)
		{
			IEntry []entries = PathMultiLookup(
				pathExpression, flags, requiredTypes);

			if (entries.Length > 1)
				throw new SingletonFailureException();

			if (entries.Length == 0)
				return null;

			return entries[0];
		}

		/// <summary>
		/// Lookup a single entry in context space (or local space).  If the
		/// srting expression given resolves to more then one entry, then an
		/// exception will be thrown indicating that the expression does not
		/// resolve to a singleton.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <param name="flags">Flags which indicate what kind of filtering
		/// should be done.</param>
		/// <param name="requiredType">A QName which inidicates which
		/// Context space entry type is permitted.  This parameter cannot be
		/// null or an ArgumentNullException will be thrown.</param>
		/// <returns>The IEntry (if any) which maches the given query.  If no
		/// entry matches, then this result will be null.</returns>
		static public IEntry PathLookup(string pathExpression,
			PathLookupFlags flags, XmlQualifiedName requiredType)
		{
			if (requiredType == null)
				throw new ArgumentNullException("requiredType",
					"\"requiredType\" parameter cannot be null.");

			return PathLookup(pathExpression, flags,
				new XmlQualifiedName[] {requiredType});
		}

		/// <summary>
		/// Lookup a single entry in context space (or local space).  If the
		/// srting expression given resolves to more then one entry, then an
		/// exception will be thrown indicating that the expression does not
		/// resolve to a singleton.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <param name="flags">Flags which indicate what kind of filtering
		/// should be done.</param>
		/// <returns>The IEntry (if any) which maches the given query.  If no
		/// entry matches, then this result will be null.</returns>
		static public IEntry PathLookup(string pathExpression,
			PathLookupFlags flags)
		{
			return PathLookup(pathExpression, flags, (XmlQualifiedName[])null);
		}

		/// <summary>
		/// Lookup a single entry in context space (or local space).  If the
		/// srting expression given resolves to more then one entry, then an
		/// exception will be thrown indicating that the expression does not
		/// resolve to a singleton.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <returns>The IEntry (if any) which maches the given query.  If no
		/// entry matches, then this result will be null.</returns>
		static public IEntry PathLookup(string pathExpression)
		{
			return PathLookup(pathExpression, PathLookupFlags.MustExist);
		}

		/// <summary>
		/// Lookup all entries in context space (or local space) that match a
		/// given query.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <param name="flags">Flags which indicate what kind of filtering
		/// should be done.</param>
		/// <param name="requiredTypes">A list of QNames which inidicate which
		/// Context space entry types are permitted.  If this parameter is null,
		/// then all types are permitted.</param>
		/// <returns>The array of IEntry (if any) which match the given query.
		/// If no entries matche, then this result will be an empty array.
		/// </returns>
		static public IEntry[] PathMultiLookup(string pathExpression,
			PathLookupFlags flags, XmlQualifiedName []requiredTypes)
		{
			bool passes;
			PathExpressionCollection paths = 
				new PathExpressionCollection(pathExpression);

			IEntry[] entries = paths.resolveAll();
			ArrayList list = new ArrayList();
			foreach (IEntry entry in entries)
			{
				if (( (flags & PathLookupFlags.MustExist) > 0)
					&& !entry.Exists)
					continue;
				if (( (flags & PathLookupFlags.Local) > 0)
					&& !entry.Local)
					continue;
				if (( (flags & PathLookupFlags.Grid) > 0)
					&& entry.Local)
					continue;

				if (requiredTypes != null)
				{
					passes = false;
					foreach (XmlQualifiedName type in requiredTypes)
					{
						if (type.Equals(entry.EntryType))
						{
							passes = true;
							break;
						}
					}

					if (!passes)
						continue;
				}

				list.Add(entry);
			}

			entries = new IEntry[list.Count];
			list.CopyTo(entries);

			return entries;
		}

		/// <summary>
		/// Lookup all entries in context space (or local space) that match a
		/// given query.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <param name="flags">Flags which indicate what kind of filtering
		/// should be done.</param>
		/// <param name="requiredType">A QNames which inidicates which
		/// Context space entry type is permitted.  This parameter cannot be
		/// null or an ArgumentNullException will be thrown.</param>
		/// <returns>The array of IEntry (if any) which match the given query.
		/// If no entries matche, then this result will be an empty array.
		/// </returns>
		static public IEntry[] PathMultiLookup(string pathExpression,
			PathLookupFlags flags, XmlQualifiedName requiredType)
		{
			if (requiredType == null)
				throw new ArgumentNullException("requiredType",
					"\"requiredType\" parameter cannot be null.");

			return PathMultiLookup(pathExpression,
				flags, new XmlQualifiedName[] { requiredType } );
		}

		/// <summary>
		/// Lookup all entries in context space (or local space) that match a
		/// given query.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <param name="flags">Flags which indicate what kind of filtering
		/// should be done.</param>
		/// <returns>The array of IEntry (if any) which match the given query.
		/// If no entries matche, then this result will be an empty array.
		/// </returns>
		static public IEntry[] PathMultiLookup(string pathExpression,
			PathLookupFlags flags)
		{
			return PathMultiLookup(pathExpression, flags, (XmlQualifiedName[])null);
		}

		/// <summary>
		/// Lookup all entries in context space (or local space) that match a
		/// given query.
		/// </summary>
		/// <param name="pathExpression">The string expression which describes
		/// this path.  This expression has the form:
		/// 
		/// <code>[local:]&lt;path&gt;</code>
		/// </param>
		/// <returns>The array of IEntry (if any) which match the given query.
		/// If no entries matche, then this result will be an empty array.
		/// </returns>
		static public IEntry[] PathMultiLookup(string pathExpression)
		{
			return PathMultiLookup(pathExpression, PathLookupFlags.MustExist);
		}
	}
}