// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;

using UVa.GCG.WSRF.Common.WS.Addressing;

using Grid.Env;
using Grid.Extensions;

namespace Grid.ContextSpace
{
	/// <summary>
	/// This class serves as a singleton location where all known context
	/// entry types are collected.  Not being in this collection doesn't
	/// necessarily mean that your type is invalid, but it means that this
	/// grid library doesn't know about it.  Right now, this collection is
	/// compile-time bound and it would be not too difficult to make it
	/// configuration time bound so I forsee changes in the very near future.
	/// </summary>
	public class EntryCollection
	{
		static private EntryCollection _instance =
			new EntryCollection();

		/// <summary>
		/// The singleton instance of this class.
		/// </summary>
		static public EntryCollection Instance
		{
			get
			{
				return _instance;
			}
		}

		/// <summary>
		/// A map of XmlQualifiedName to IEntryFactory instances.
		/// </summary>
		private Hashtable _factories = new Hashtable();

		private EntryCollection()
		{
			add(new GridContextFactory());
			add(new GridFileFactory());
			add(new UnknownFactory());
			add(new GridFactoryFactory());

			addExtensions();
		}

		private void add(IEntryFactory factory)
		{
			_factories[factory.EntryType] = factory;
		}

		private void addExtensions()
		{
			ExtensionLibraryDescription []libraries =
				ExtensionCollection.Instance.InstalledExtensionLibraries;
			foreach (ExtensionLibraryDescription desc in libraries)
			{
				try
				{
					IEntryFactory []factories = desc.EntryFactories;
					foreach (IEntryFactory factory in factories)
					{
						GridLibEnvironment.Instance.Console.Trace.WriteLine(
							"Adding entry factory for type:  {0}", factory.EntryType);

						add(factory);
					}
				}
				catch (Exception e)
				{
					GridLibEnvironment.Instance.Console.Debug.WriteLine(
						"An exception occurred loading an extension library:  {0}",
						e.Message);
					continue;
				}
			}
		}

		/// <summary>
		/// Create a new IEntry given a qname for a type and
		/// other information about the entry such as EPR, etc.
		/// </summary>
		/// <param name="type">The type of the entry to create.</param>
		/// <param name="name">The name that the new entry will have (this
		/// is usually it's name in context space).</param>
		/// <param name="epr">The EPR (if available) for the backend service
		/// endpoint.  This parameter can be null if the entry doesn't refer
		/// to a WSRF endpoint (such as a local file or directory).</param>
		/// <param name="parent">The parent entry for this one.  This parameter
		/// can be null indiciating that no parent is known (note that this
		/// does not necessarily mean that the entry doesn't hav a parent, only
		/// that we don't know who the parent is).</param>
		/// <returns>A new IEntry describing the endpoint or local entry
		/// given.  This result should not be null.  Also, note that the
		/// QName type of the resultant entry may not in fact match the
		/// type given.  This can happen either when the type given was
		/// a null, or more likely, when the type given is not known to the
		/// grid library.</returns>
		public IEntry create(XmlQualifiedName type,
			string name,
			EndpointReferenceType epr,
			IEntry parent)
		{
			IEntryFactory factory;

			if (type == null)
				factory = new UnknownFactory();
			else
				factory = _factories[type] 
					as IEntryFactory;
			if (factory == null)
				factory = new UnknownFactory();

			return factory.create(name, epr, parent);
		}

		/// <summary>
		/// Find the factory registered for the given entry type.
		/// This result may be null indicating that no factory is
		/// known which matches that type.
		/// </summary>
		public IEntryFactory this[XmlQualifiedName qName]
		{
			get
			{
				return _factories[qName] as IEntryFactory;
			}
		}
	}
}