// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

using Grid.Proxies;

using UVa.GCG.WSRF.Common.WS;
using UVa.GCG.WSRF.Common.WS.Addressing;
using UVa.GCG.WSRF.Common.WS.ResourceProperties;

namespace Grid.ContextSpace
{
	/// <summary>
	/// A FactoryEntry is an IEntry which represents a type that can create
	/// new service endpoints.  This doesn't necessarily have to correspond
	/// to endpoints whose types are known.  Only that they can be created
	/// given the Factory port type interface.
	/// </summary>
	public class FactoryEntry : AbstractGridEntry
	{
		/// <summary>
		/// Construct a new FactoryEntry
		/// </summary>
		/// <param name="epr">The EPR of the factory endpoint.  This
		/// parameter should NOT be null.</param>
		/// <param name="name">The name the factory entry has in context
		/// space.</param>
		/// <param name="parent">The parent, if available, of this
		/// entry.</param>
		public FactoryEntry(
			EndpointReferenceType epr,
			string name,
			IEntry parent)
			: base(epr, name, WellknownEntryTypes.ENTRY_TYPE_FACTORY, parent)
		{
		}

		/// <summary>
		/// All factories by definition have the ability to create.  This will
		/// cause the create method on the factory port type to run and return
		/// a resultant EPR.
		/// </summary>
		/// <param name="creationHints">Creation hints that are to be given
		/// to the factory when it creates the new endpoint.  This parameter
		/// is allowed to be null or empty indicating that no creation hints
		/// are desired.</param>
		/// <returns>The new EPR for the newly created service endpoint.
		/// </returns>
		public EndpointReferenceType create(XmlElement []creationHints)
		{
			FactoryServiceProxy proxy = new FactoryServiceProxy(EPR);
			return proxy.create(creationHints);
		}

		/// <summary>
		/// All factories by definition have the ability to create.  This will
		/// cause the create method on the factory port type to run and return
		/// a resultant EPR.
		/// </summary>
		/// <param name="creationHints">Creation hints that are to be given
		/// to the factory when it creates the new endpoint.  This parameter
		/// is allowed to be null or empty indicating that no creation hints
		/// are desired.  Otherwise, it is a hashtable which maps
		/// QNames (XmlQualifiedName) to some kind of XmlSerializable type.
		/// </param>
		/// <returns>The new EPR for the newly created service endpoint.
		/// </returns>
		public EndpointReferenceType create(Hashtable creationHints)
		{
			return create(
				formatCreationHints(creationHints));
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.format"/>
		/// </summary>
		public override LsFormat format(bool isLong)
		{
			string desc = "[FACTORY]";

			if (isLong)
				desc = string.Format("[FACTORY:  {0}]",
					CreationTypeDescription);

			return new LsFormat(EntryName, desc);
		}

		/// <summary>
		/// All factories by definition have the ability to create.  When they
		/// do so, the are allowed to take creationHints to indicate to them
		/// any external creation requirements which may be appropriate.  This
		/// function takes the hashtable version of those (which is a hashtable
		/// that maps QNames (XmlQualifiedName) to some kind of XmlSerializable
		/// type) and serializes them into the XmlElement array which goes out
		/// on the wire.
		/// </summary>
		/// <param name="creationHints">Creation hints that are to be given
		/// to the factory when it creates the new endpoint.  This parameter
		/// is allowed to be null or empty indicating that no creation hints
		/// are desired.  Otherwise, it is a hashtable which maps
		/// QNames (XmlQualifiedName) to some kind of XmlSerializable type.
		/// </param>
		/// <returns>The new XmlElement array which is the serialized version
		/// of the creationHints table.
		/// </returns>
		static public XmlElement[] formatCreationHints(Hashtable creationHints)
		{
			if (creationHints == null)
				return null;
				
			XmlDocument doc = new XmlDocument();
			XmlElement []hints = new XmlElement[creationHints.Count];

			int lcv = 0;
			foreach (XmlQualifiedName qName in creationHints.Keys)
			{
				XmlRootAttribute root = new XmlRootAttribute(qName.Name);
				root.Namespace = qName.Namespace;
				hints[lcv++] = WSUtilities.Serialize(
					creationHints[qName], root);
			}

			return hints;
		}

		/// <summary>
		/// Cached value of the creation type property.
		/// </summary>
		private XmlQualifiedName _creationType = null;

		/// <summary>
		/// A property which allows you to ask the factory entry type
		/// what context space type this factory creates.  This parameter
		/// can be cashed up, but is not guaranteed to be so and may cause
		/// an outcall to take place.
		/// </summary>
		public XmlQualifiedName CreationType
		{
			get
			{
				lock (this)
				{
					if (_creationType == null)
						_creationType = getCreationType(EPR);

					return _creationType;
				}
			}
		}

		/// <summary>
		/// A simple property which uses the creation type to lookup a
		/// more readable creation type description.  This is typically
		/// used during long ls listings to give information about what
		/// this factory type is used for.
		/// </summary>
		public string CreationTypeDescription
		{
			get
			{
				string desc = null;
				IEntryFactory fact = EntryCollection.Instance[CreationType];

				if (fact != null)
					desc = EntryCollection.Instance[CreationType].CommonName;

				if (desc == null)
					desc = "<unknown>";

				return desc;
			}
		}

		/// <summary>
		/// This method attempts to determine what the creation type of a
		/// given EPR is.
		/// </summary>
		/// <param name="epr">The EPR of a factory type for which we wish
		/// to find it's creation type.  A couple of things are assumed 
		/// about the EPR are assumed.  We assume that the EPR refers to a valid,
		/// real factory type and that that factory type has the required
		/// "factory-type" resource property.  If this is not true, exceptions
		/// will be thrown.</param>
		/// <returns>The QName of the context space type which this factory
		/// creates.</returns>
		static private XmlQualifiedName getCreationType(EndpointReferenceType epr)
		{
			// Factory type is given by a resource property, so we need to
			// use the GetResourceProperty port type to find this factory type
			// element.
			GetResourcePropertyBinding proxy =
				new GetResourcePropertyBinding(epr);

			GetResourcePropertyResponse resp = null;
			
			try
			{
				resp = proxy.GetResourceProperty(
					new XmlQualifiedName(
					GridConstants.GRID_FACTORY_TYPE_ELEMENT,
					GridConstants.GRID_NS));
			}
			catch (Exception)
			{
			}

			// If we got any kind of exception at all, then things are bad, but
			// we tye to get on with life by merely saying we don't know the
			// type.  Later, we may run into trouble, but we'll wait 'til then.
			if (resp == null)
				return new XmlQualifiedName(
					"<unknown>", GridConstants.GRID_NS);

			// Since we got a response, let's parse out the qname (which in the
			// property is merely given as it's string representation.
			string str = (resp.Any[0] as XmlElement).InnerText;
			int index = str.LastIndexOf(':');
			return new XmlQualifiedName(
				str.Substring(index + 1),
				str.Substring(0, index));
		}
	}
}