// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.WS;
using UVa.GCG.WSRF.Common.WS.Addressing;
using UVa.GCG.WSRF.Common.WS.ResourceProperties;
using UVa.GCG.WSRF.Common.WS.ServiceGroup;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An entry type for any type of context which is in the grid (i.e., not
	/// for local directories).
	/// </summary>
	public class GridContext : AbstractGridEntry, IContext
	{
		/// <summary>
		/// Create a new GridContext entry.
		/// </summary>
		/// <param name="epr">The EPR of the endpoint that this entry
		/// represents.</param>
		/// <param name="name">The name of the context entry.</param>
		/// <param name="parent">The parent entry for this IENtry type
		/// (if available).</param>
		public GridContext(EndpointReferenceType epr, string name, IEntry parent)
			: base(epr, name, WellknownEntryTypes.ENTRY_TYPE_CONTEXT, parent)
		{
		}

		#region IContext Members

		/// <summary>
		/// Cached children entry.
		/// </summary>
		private IEntry[] _children = null;

		/// <summary>
		/// <see cref="Grid.ContextSpace.IContext.Children"/>
		/// </summary>
		public IEntry[] Children
		{
			get
			{
				lock (this)
				{
					if (_children == null)
						_children = getChildren();

					return _children;
				}
			}
		}

		/// <summary>
		/// <see cref="IContext.this[string]"/>
		/// </summary>
		public IEntry this[string child]
		{
			get
			{
				foreach (IEntry entry in Children)
				{
					if (entry.EntryName.Equals(child))
						return entry;
				}

				return null;
			}
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IContext.createChild"/>
		/// </summary>
		public IEntry createChild(XmlQualifiedName childType, string child,
			Hashtable creationHints)
		{
			Hashtable hints;
			if (creationHints == null)
				hints = new Hashtable();
			else
			{
				// We clone this because we may make some modifications to
				// it as we go.
				hints = (Hashtable)creationHints.Clone();
			}

			// Locate the factory that creates context space elements.  This
			// could be specified in the hints, or it could be located as the
			// default entry in the Factories context.
			FactoryEntry factory = findFactory(childType, hints);
			if (factory == null)
				throw new NoSuitableFactoryException(childType);

			// Ask the factory to create a new instance of the correct type.
			EndpointReferenceType epr = factory.create(
				FactoryEntry.formatCreationHints(hints));

			// Now that we have the created instance, lets link it into
			// context space.
			IEntry entry = link(childType, child, epr, true);
			if (childType.Equals(WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
			{
				IContext c = (IContext)entry;
				c.link(WellknownEntryTypes.ENTRY_TYPE_CONTEXT, ".", epr, true);
				c.link(WellknownEntryTypes.ENTRY_TYPE_CONTEXT, "..", EPR, true);
			}

			return entry;
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IContext.link"/>
		/// </summary>
		public IEntry link(XmlQualifiedName entryType,
			string name, EndpointReferenceType epr, bool hardLink)
		{
			if (!hardLink)
				throw new UnimplementedException(
					this.GetType(), this.GetType().GetMethod("link",
					new Type[] {entryType.GetType(), name.GetType(),
								   epr.GetType(), hardLink.GetType()}),
					"Soft links are not yet implemented.");

			// You can only link in child elements using this link method.
			// If you are looking to link in a full path, then use the more
			// generic GridLnOperation class (which will inevitably call 
			// this method on the correct context entry).
			if (name.IndexOf("/") >= 0)
				throw new InvalidPathException(name);

			// Go ahead and add the entry to this context (service group).
			addEntry(
				EPR, name,
				epr, entryType);

			// Finally, return a new IEntry of the correct type.
			return EntryCollection.Instance.create(
				entryType, name, epr, this);
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IContext.remove"/>
		/// </summary>
		public IEntry remove(string child)
		{
			throw new UnimplementedException(
				this.GetType(), this.GetType().GetMethod("remove"));
		}

		/// <summary>
		/// <see cref="IEntry.format"/>
		/// </summary>
		public override LsFormat format(bool isLong)
		{
			return new LsFormat(EntryName,
				"[DIR]");
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IContext.mkdir"/>
		/// </summary>
		public IEntry mkdir(string child, Hashtable creationHints)
		{
			return createChild(
				WellknownEntryTypes.ENTRY_TYPE_CONTEXT, child, creationHints);
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IContext.touch"/>
		/// </summary>
		public IEntry touch(string child, Hashtable creationHints)
		{
			return createChild(
				WellknownEntryTypes.ENTRY_TYPE_FILE, child, creationHints);
		}

		#endregion

		#region Private Members

		/// <summary>
		/// Find a factory port type suitable for creating the indicated
		/// type of entry.
		/// </summary>
		/// <param name="type">The QName of the entry type that we would
		/// like to create.</param>
		/// <param name="creationHints">A hashtable which maps QNames to
		/// XML Serializable instances that represent hints which may affect
		/// creation.  For this method, the hint that we are looking for is:
		/// 
		/// <code>
		/// http://cs.virginia.edu/gbg:creation-factory
		/// </code>
		/// 
		/// </param>
		/// <returns>An entry that describes the factory type that we need
		/// a reference to.</returns>
		static private FactoryEntry findFactory(XmlQualifiedName type,
			Hashtable creationHints)
		{
			EndpointReferenceType factory;

			// First, let's see if they specified their own factory.
			factory = creationHints[WellknownCreationHints.CREATION_FACTORY]
				as EndpointReferenceType;
			if (factory != null)
			{
				// If so, we'll use that one (and remove it so that it doesn't
				// have to go out on the wire).
				creationHints.Remove(WellknownCreationHints.CREATION_FACTORY);
				return new FactoryEntry(factory, "<unknown>", null);
			}

			// If they didn't specify a factory, we need to try and find one
			// that is appropriate.  To do this, we look in the well known
			// system path for factories.
			IEntry []entries = ContextUtilities.PathMultiLookup("/System/Factories/*",
				PathLookupFlags.Grid | PathLookupFlags.MustExist,
				WellknownEntryTypes.ENTRY_TYPE_FACTORY);

			foreach (FactoryEntry entry in entries)
			{
				if (entry.CreationType.Equals(type))
					return entry;
			}

			// If we get here, we couldn't find a factory.  Don't know what
			// this will mean to our caller, but we'll let them decide.
			return null;
		}

		private IEntry[] getChildren()
		{
			ArrayList entries = readEntries();
			IEntry []ret = new IEntry[entries.Count];
			entries.CopyTo(ret);

			return ret;
		}

		/// <summary>
		/// This is one of the work-horse methods for contexts.  It
		/// asks the context for all of it's children.  In the future,
		/// it may make sense to try and filter out entries we don't
		/// care about, but until then, this method will suffice.
		/// </summary>
		/// <returns>An ArrayList of IEntry elements which represents
		/// the children of this context.</returns>
		private ArrayList readEntries()
		{
			EndpointReferenceType context = EPR;
			ArrayList list = new ArrayList();
			XmlRootAttribute rootAttr =
				new XmlRootAttribute("Entry");
			rootAttr.Namespace = WSConstants.WS_SERVICE_GROUPS_XSD_NS;

			QueryResourcePropertiesRequest query =
				new QueryResourcePropertiesRequest();
			query.QueryExpression = new QueryExpressionType();
			query.QueryExpression.dialect = GridConstants.XPATH_DIALECT;

			string xpathQuery = string.Format(
				"/*/*[namespace-uri()='{0}' and local-name()='Entry' "
				+ "and *[namespace-uri()='{0}' and" +
				" local-name()='Content' and *[namespace-uri()='{1}' "
				+ "and local-name()='{2}']]]",
				WSConstants.WS_SERVICE_GROUPS_XSD_NS,
				GridConstants.GRID_NS,
				GridConstants.ENTRY_NAME_NAME);
			XmlDocument doc = new XmlDocument();
			query.QueryExpression.Any = new XmlNode[]
				{
					doc.CreateTextNode(xpathQuery)
				};

			QueryResourcePropertiesBinding tsgService =
				new QueryResourcePropertiesBinding(context);
			QueryResourcePropertiesResponse response =
				tsgService.QueryResourceProperties(query);

			if (response != null && response.Any != null)
			{
				foreach (XmlElement element in response.Any)
				{
					XmlElement content = element["Content",
						WSConstants.WS_SERVICE_GROUPS_XSD_NS];
					if (content == null)
						continue;

					ServiceGroupContentType sgct =
						WSUtilities.Deserialize(content,
						typeof(ServiceGroupContentType))
						as ServiceGroupContentType;
					if (sgct == null)
						continue;

					content = element["MemberServiceEPR",
						WSConstants.WS_SERVICE_GROUPS_XSD_NS];
					if (content == null)
						continue;

					XmlRootAttribute root = new XmlRootAttribute(
						"MemberServiceEPR");
					root.Namespace = WSConstants.WS_SERVICE_GROUPS_XSD_NS;

					EndpointReferenceType epr =
						WSUtilities.Deserialize(content,
						typeof(EndpointReferenceType), root) as
						EndpointReferenceType;

					if (epr == null)
						continue;

					list.Add(
						EntryCollection.Instance.create(
						new XmlQualifiedName(
							sgct.EntryType.Name, sgct.EntryType.NamespaceURI),
						sgct.EntryName, epr, this));
				}
			}

			return list;
		}

		static private void addEntry(EndpointReferenceType context, string entryName,
			EndpointReferenceType entry, XmlQualifiedName entryType)
		{
			AddType add = new AddType();
			add.Content = new XmlElement[]
				{
					createNameElement(entryName),
					createTypeElement(entryType)
				};
			add.MemberEPR = entry;
			add.InitialTerminationTimeSpecified = false;
			ServiceGroupRegistrationBinding sg =
				new ServiceGroupRegistrationBinding(context);
			sg.Add(add);
		}

		static private XmlElement createNameElement(string name)
		{
			XmlDocument doc = new XmlDocument();
			doc.AppendChild(doc.CreateElement(
				GridConstants.ENTRY_NAME_NAME, GridConstants.GRID_NS));
			doc.DocumentElement.InnerText = name;
			return doc.DocumentElement;
		}

		static private XmlElement createTypeElement(XmlQualifiedName entryType)
		{
			XmlDocument doc = new XmlDocument();
			doc.AppendChild(doc.CreateElement(
				GridConstants.ENTRY_TYPE_NAME,
				GridConstants.GRID_NS));
			doc.DocumentElement.AppendChild(
				doc.CreateElement(
				entryType.Name, entryType.Namespace));
			return doc.DocumentElement;
		}

		#endregion
	}
}
