// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.WS;
using UVa.GCG.WSRF.Common.WS.Addressing;

namespace Grid.ContextSpace
{
	/// <summary>
	/// This class is used to represent absolutely ANY endpoint.  It can be
	/// an Endpoint in context space, an arbitrary EPR, etc.
	/// </summary>
	public class GridEndpointDescription
	{
		private const string EPR_FILE_HEADER = "epr-file:";

		private IEntry _entry;

		/// <summary>
		/// Construct a new endpoint description.  This constrcutor is
		/// private because we want you to use the factory methods provided
		/// instead to create instances.
		/// </summary>
		/// <param name="entry">The entry for this description.</param>
		private GridEndpointDescription(IEntry entry)
		{
			_entry = entry;
		}

		/// <summary>
		/// Retrieve the IEntry for the endpoint described here.  Note that
		/// the entry returned could be VERY generic.  In fact, it is quite
		/// possible that the entry will refer to an endpoint whose type
		/// could be known, but currently isn't.  This is to avoid unnecessary
		/// out calls if the intention is merely to have the EPR, but it
		/// allows for additional information to be available if possible.
		/// </summary>
		public IEntry Entry
		{
			get
			{
				return _entry;
			}
		}

		/// <summary>
		/// A factory method which creates new GridEndpointDescription
		/// instances that represent an already known entry.
		/// </summary>
		/// <param name="entry">The entry to create a description
		/// for.</param>
		/// <returns>A new GridEndpointDescripton</returns>
		static public GridEndpointDescription create(
			IEntry entry)
		{
			return new GridEndpointDescription(entry);
		}

		/// <summary>
		/// A factory method which creates new GridEndpointDescription
		/// instances that represent some entry that is being described
		/// in text.
		/// </summary>
		/// <param name="endpointDescription">The string description of
		/// some entry.  The format of this string is as follows:
		/// 
		/// <code>
		///		[epr-file:]&lt;some-path-to-file&gt;
		///			OR
		///		&lt;grid-path&gt;
		/// </code>
		/// 
		/// </param>
		/// <param name="type">The type of the new entry (if the type
		/// is known).  This parameter can be null if unknown.</param>
		/// <returns>A new GridEndpointDescripton</returns>
		static public GridEndpointDescription create(
			string endpointDescription, XmlQualifiedName type)
		{
			if (endpointDescription.StartsWith(
				EPR_FILE_HEADER))
				return createEndpointFromFile(endpointDescription.Substring(
					EPR_FILE_HEADER.Length), type);
			else
				return createEndpointFromPath(
					endpointDescription);
		}

		/// <summary>
		/// A factory method which creates new GridEndpointDescription
		/// instances that represent some entry that is being described
		/// in text.
		/// </summary>
		/// <param name="endpointDescription">The string description of
		/// some entry.  The format of this string is as follows:
		/// 
		/// <code>
		///		[epr-file:]&lt;some-path-to-file&gt;
		///			OR
		///		&lt;grid-path&gt;
		/// </code>
		/// 
		/// </param>
		/// <returns>A new GridEndpointDescripton</returns>
		static public GridEndpointDescription create(
			string endpointDescription)
		{
			return create(endpointDescription, WellknownEntryTypes.ENTRY_TYPE_UNKNOWN);
		}

		/// <summary>
		/// Creates a new GridEndpointDescription by reading an EPR (in XML
		/// form) out of a file (either local, or grid).
		/// </summary>
		/// <param name="filePath">The path (local, or grid) to a file whic
		/// contains an XML Serialized form of an EPR</param>
		/// <param name="type">The type if known.  This parameter is allowed
		/// to be null.</param>
		/// <returns>A new GridEndpointDescription instance.</returns>
		static public GridEndpointDescription
			createEndpointFromFile(string filePath, XmlQualifiedName type)
		{
			EndpointReferenceType epr;
			IEntry file = ContextUtilities.PathLookup(filePath,
				PathLookupFlags.MustExist,
				WellknownEntryTypes.ENTRY_TYPE_FILE);

			if (file == null)
				throw new NoSuchPathException(filePath);

			IFile f = (IFile)file;

			using (Stream stream = f.OpenRead())
			{
				XmlRootAttribute root = new XmlRootAttribute(
					GridConstants.EPR_INFO_ROOT);
				root.Namespace = GridConstants.GRID_NS;

				XmlSerializer ser = new XmlSerializer(
					typeof(EndpointReferenceType), root);
				epr = (EndpointReferenceType)ser.Deserialize(stream);
			}

			return new GridEndpointDescription(
				EntryCollection.Instance.create(type, null, epr, null));
		}

		/// <summary>
		/// Create a new GridEndpointDescription from a path in context space
		/// to an entry.
		/// </summary>
		/// <param name="path">The path in context space which points to an
		/// entry.  This path is assumed to be a grid-path and not to refer
		/// to a local file or directory.</param>
		/// <returns>A new GridEndpointDescription instance.</returns>
		static public GridEndpointDescription
			createEndpointFromPath(string path)
		{
			return new GridEndpointDescription(
				ContextUtilities.PathLookup(path,
					PathLookupFlags.Grid | PathLookupFlags.MustExist));
		}
	}
}