// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;

using UVa.GCG.WSRF.Common.WS;
using UVa.GCG.WSRF.Common.WS.Addressing;
using UVa.GCG.WSRF.Common.WS.ResourceProperties;

using Grid.Proxies;

namespace Grid.ContextSpace
{
	/// <summary>
	/// Reperesents entries in context space which are files inside of the
	/// grid.
	/// </summary>
	public class GridFile : AbstractGridEntry, IFile
	{
		/// <summary>
		/// Construct a new GridFile.
		/// </summary>
		/// <param name="epr">The endpoint reference of the entry for which
		/// this GridFile is an entry.  This parameter must not be null.
		/// </param>
		/// <param name="entryName">The name of this entry in context
		/// space.</param>
		/// <param name="parent">The parent, if known, of this entry.  This
		/// parameter is allowed to be null if unavailable or unknown.</param>
		public GridFile(EndpointReferenceType epr,
			string entryName,
			IEntry parent)
			: base(epr, entryName, WellknownEntryTypes.ENTRY_TYPE_FILE, parent)
		{
		}

		/// <summary>
		/// <see cref="Grid.ContextSpace.IEntry.format"/>
		/// </summary>
		public override LsFormat format(bool isLong)
		{
			if (isLong)
				return new LsFormat(EntryName,
					string.Format("[FILE:  {0} bytes]", Size));
			else
				return new LsFormat(EntryName);
		}

		#region IFile Members

		/// <summary>
		/// The cached size information for a file.  If this value is -1,
		/// then the size is unknown and can't be known for some reason.
		/// If it is -2, then it means that it is unknown but we haven't
		/// yet tried to figure it out (i.e., it is not in the cache).
		/// </summary>
		private long _size = -2;

		/// <summary>
		/// <see cref="Grid.ContextSpace.IFile.Size"/>
		/// </summary>
		public long Size
		{
			get
			{
				if (_size < -1)
					_size = getSize();

				return _size;
			}
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.read"/></remarks>
		public byte[] read(long offset, int length)
		{
			FileServiceProxy proxy =
				new FileServiceProxy(EPR);
			return Convert.FromBase64String(
				proxy.read(offset, length));
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.write"/></remarks>
		public int write(long offset, byte[] data)
		{
			FileServiceProxy proxy =
				new FileServiceProxy(EPR);
			return proxy.write(offset, Convert.ToBase64String(data));
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.truncAppend"/>
		/// </remarks>
		public int truncAppend(byte[] data)
		{
			FileServiceProxy proxy =
				new FileServiceProxy(EPR);
			return proxy.truncAppend(Convert.ToBase64String(data));
		}

		/// <remarks><see cref="IFile.append"/></remarks>
		public int append(byte []data)
		{
			FileServiceProxy proxy =
				new FileServiceProxy(EPR);
			return proxy.append(Convert.ToBase64String(data));
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.OpenRead"/></remarks>
		public Stream OpenRead()
		{
			return new GridFileStream(this,
				FileMode.Open, FileAccess.Read);
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.OpenWrite"/></remarks>
		public Stream OpenWrite()
		{
			return new GridFileStream(this,
				FileMode.OpenOrCreate, FileAccess.Write);
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.Open"/></remarks>
		public Stream Open(FileMode mode)
		{
			return new GridFileStream(this, mode,
				FileAccess.ReadWrite);
		}

		/// <remarks><see cref="Grid.ContextSpace.IFile.Open"/></remarks>
		public Stream Open(FileMode mode, FileAccess access)
		{
			return new GridFileStream(this, mode, access);
		}

		#endregion

		#region Private Members

		private long getSize()
		{
			GetResourcePropertyBinding proxy =
				new GetResourcePropertyBinding(EPR);
			GetResourcePropertyResponse resp =
				proxy.GetResourceProperty(
					GridConstants.FILE_SIZE_PROPERTY);
			if (resp == null || resp.Any == null || resp.Any.Count != 1)
				return -1;

			return long.Parse(((XmlElement)(resp.Any[0])).InnerText);
		}

		#endregion
	}
}
