// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;

using Grid.Proxies;

namespace Grid.ContextSpace
{
	/// <summary>
	/// The GridFileStream class is a class, derived off of 
	/// <see cref="System.IO.Stream"/> which attempts to duplicate all
	/// of the stream like behaviors of a local file.  This class is
	/// technically correct, but it needs some work on better caching,
	/// pre-fetching, asynchronous IO, and block reading.
	/// </summary>
	public class GridFileStream : Stream
	{
		private GridFile _gridFile;
		private long _minimumPosition = 0;
		private FileAccess _access;
		private bool _nextIsTruncate = false;
		private long _length = -1;
		private long _position = 0;
		private FileServiceProxy _proxy = null;

		/// <summary>
		/// Construct a new GridFileStream which represents a
		/// given GridFile entry.  This constructor assumes a
		/// FileMode of <see cref="System.IO.FileMode.OpenOrCreate"/>
		/// and a FileAccess of <see cref="System.IO.FileAccess.ReadWrite"/>
		/// </summary>
		/// <param name="gridFile">The GridFile which this stream
		/// is to be connected to.</param>
		public GridFileStream(GridFile gridFile)
			: this(gridFile, FileMode.OpenOrCreate)
		{
		}

		/// <summary>
		/// Construct a new GridFileStream which represents a
		/// given GridFile entry.  This constructor assumes a
		/// FileAccess of <see cref="System.IO.FileAccess.ReadWrite"/>
		/// </summary>
		/// <param name="gridFile">The GridFile which this stream
		/// is to be connected to.</param>
		/// <param name="mode">The file mode to open this file with.</param>
		public GridFileStream(GridFile gridFile,
			FileMode mode)
			: this(gridFile, mode, FileAccess.ReadWrite)
		{
		}

		/// <summary>
		/// Construct a new GridFileStream which represents a
		/// given GridFile entry.</summary>
		/// <param name="gridFile">The GridFile which this stream
		/// is to be connected to.</param>
		/// <param name="mode">The file mode to open this file with.</param>
		/// <param name="access">The file access to use when opening this
		/// file.</param>
		public GridFileStream(GridFile gridFile,
			FileMode mode, FileAccess access)
		{
			_gridFile = gridFile;
			_access = access;

			// If they requested an operation which should truncate the
			// grid file, let's make sure that the next operation we perform
			// that writes to the file does in fact truncate it.
			if ( (mode & FileMode.Truncate) > 0 )
				_nextIsTruncate = true;
			if ( (mode & FileMode.Create) > 0)
				_nextIsTruncate = true;

			_proxy = new FileServiceProxy(gridFile.EPR);

			// If they opened the file in append mode, we keep
			// track of the original length of the file so that they
			// can't accidentally seek back before that original length.
			if ( (mode & FileMode.Append) > 0)
			{
				_minimumPosition = Length;
				_position = _minimumPosition;
			}
		}

		/// <summary><see cref="System.IO.Stream.CanRead"/></summary>
		public override bool CanRead
		{
			get
			{
				return ((_access & FileAccess.Read) > 0) ||
					((_access & FileAccess.ReadWrite) > 0);
			}
		}

		/// <summary><see cref="System.IO.Stream.CanSeek"/></summary>
		public override bool CanSeek
		{
			get
			{
				return true;
			}
		}

		/// <summary><see cref="System.IO.Stream.CanWrite"/></summary>
		public override bool CanWrite
		{
			get
			{
				return ((_access & FileAccess.Write) > 0) ||
					((_access & FileAccess.ReadWrite) > 0);
			}
		}

		/// <summary><see cref="System.IO.Stream.Flush"/></summary>
		public override void Flush()
		{
			if (_nextIsTruncate)
			{
				_proxy.truncAppend(Convert.ToBase64String(new byte[0], 0, 0));
				_nextIsTruncate = false;
			}
		}

		/// <summary><see cref="System.IO.Stream.Length"/></summary>
		public override long Length
		{
			get
			{
				if (_length < 0)
					_length = _gridFile.Size;

				return _length;
			}
		}

		/// <summary><see cref="System.IO.Stream.Position"/></summary>
		public override long Position
		{
			get
			{
				return _position;
			}

			set
			{
				if (_minimumPosition > value)
					throw new IOException(
						"Can't reposition beyond append point.");
				_position = value;
			}
		}

		/// <remarks><see cref="System.IO.Stream.Read"/></remarks>
		public override int Read(byte[] buffer, int offset, int count)
		{
			if (!CanRead)
				throw new IOException(
					"Unable to read from a file opened for write.");

			string data = _proxy.read(_position, count);
			byte []result = Convert.FromBase64String(data);
			Array.Copy(result, 0, buffer, offset, result.Length);

			_position += result.Length;
			return result.Length;
		}

		/// <remarks><see cref="System.IO.Stream.Write"/></remarks>
		public override void Write(byte[] buffer, int offset, int count)
		{
			int written;

			if (!CanWrite)
				throw new IOException(
					"Unable to read from a file opened for write.");

			if (_nextIsTruncate)
			{
				written = _proxy.truncAppend(
					Convert.ToBase64String(buffer, offset, count));
				_nextIsTruncate = false;
			} 

			else
			{
				written = _proxy.write(
					_position, Convert.ToBase64String(buffer, offset, count));
			}

			if (written != count)
				throw new ShortWriteException();

			_position += written;
			_length = 
				(_position > _length) ? _position : _length;
		}

		/// <remarks><see cref="System.IO.Stream.Seek"/></remarks>
		public override long Seek(long offset, SeekOrigin origin)
		{
			long _newPosition = _position;

			switch (origin)
			{
				case SeekOrigin.Begin :
					_newPosition = offset;
					break;
				case SeekOrigin.Current :
					_newPosition += offset;
					break;
				case SeekOrigin.End :
					_newPosition = _length += offset;
					break;
			}

			if (_newPosition < _minimumPosition)
				throw new IOException(
					"Can't reposition beyond append point.");

			return _position;
		}

		/// <remarks><see cref="System.IO.Stream.SetLength"/></remarks>
		public override void SetLength(long value)
		{
			throw new UnimplementedException(
				this.GetType(), this.GetType().GetMethod("SetLength"));
		}
	}
}