// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;

using UVa.GCG.WSRF.Common.WS.Addressing;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An IEntry type which represents the fact that a path referred to an
	/// entry which was definitely in context space, but which doesn't
	/// actually exist.  The only useful action that can be performed with
	/// an instance of this type is to create it as either a file or a
	/// new context.
	/// </summary>
	public class GridNotAvailableEntry : AbstractGridEntry, INotAvailableEntry
	{
		/// <summary>
		/// Construct a new GridNotAvailableEntry.
		/// </summary>
		/// <param name="entryName">The name of this entry in context space.
		/// </param>
		/// <param name="parent">The parent of this entry.  Unlike most
		/// IEntry instances, this parameter is required in this case as it's
		/// parent will be used to construct a new directory or file for this
		/// entry.</param>
		public GridNotAvailableEntry(string entryName, IEntry parent)
			: base(null, entryName, null, parent)
		{
			if (parent == null)
				throw new ArgumentNullException("parent",
					"The parent entry for a " +
					"GridNotAvailableEntry cannot be null.");

			if (!parent.Exists &&
				!parent.EntryType.Equals(
					WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
				throw new ArgumentException("The parent of a " +
					"GridNotAvailableEntry must either be a context, or must\n"
					+ "not exist.", "parent");
		}

		/// <remarks><see cref="Grid.ContextSpace.IEntry.format"/></remarks>
		public override LsFormat format(bool isLong)
		{
			throw new UnimplementedException(
				this.GetType(), this.GetType().GetMethod("format"));
		}

		#region Not Available Methods

		/// <remarks>
		/// <see cref="Grid.ContextSpace.INotAvailableEntry.createDirectory"/>
		/// </remarks>
		public IEntry createDirectory(bool createParents, Hashtable creationHints)
		{
			return create(createParents,
				WellknownEntryTypes.ENTRY_TYPE_CONTEXT,
				creationHints);
		}

		/// <remarks>
		/// <see cref="Grid.ContextSpace.INotAvailableEntry.createFile"/>
		/// </remarks>
		public IEntry createFile(bool createParents, Hashtable creationHints)
		{
			return create(createParents,
				WellknownEntryTypes.ENTRY_TYPE_FILE,
				creationHints);
		}

		/// <remarks>
		/// <see cref="Grid.ContextSpace.INotAvailableEntry.create"/>
		/// </remarks>
		public IEntry create(bool createParents, XmlQualifiedName type,
			Hashtable creationHints)
		{
			IEntry parent = Parent;

			// While we absolutely require that our parent entry is non-null
			// that doesn't mean that it represent a context that in fact
			// exists.
			if (!parent.Exists)
			{
				// If it doesn't exist, but we can't create the parents, then
				// we have to exit exceptionally.
				if (!createParents)
					throw new NoSuchPathException(
						parent.Path);

				// If we are allowed to create our parents, then do so.
				INotAvailableEntry p = (INotAvailableEntry)parent;
				parent = p.createDirectory(createParents, creationHints);
			}

			// Nwo that we have a parent, use that parent to create us.
			IContext c = (IContext)parent;
			return c.createChild(type, EntryName, creationHints);
		}

		#endregion
	}
}