// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;

using UVa.GCG.WSRF.Common.WS.Addressing;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An interface which extends IEntry to provide functionallity
	/// common to contexts.
	/// </summary>
	public interface IContext : IEntry
	{
		/// <summary>
		/// Retrieves the list of children for this context.
		/// </summary>
		IEntry[] Children { get; }

		/// <summary>
		/// Looks up a child with a given name inside this
		/// context.
		/// </summary>
		IEntry this[string child] { get; }

		/// <summary>
		/// Create a new child inside of this context.
		/// </summary>
		/// <param name="childType">The QName of the child
		/// type that we wish to create.  This type must be non-null but
		/// it does not infact have to be recognized as a well known
		/// entry type.  However, we have to be able to find a factory
		/// that supports this child type.  This can be done either
		/// by supplying a factory in the creation hints, or if it
		/// can be found in /System/Factories, then that factory can
		/// be used.</param>
		/// <param name="child">The name of the child we wish to create.
		/// </param>
		/// <param name="creationHints">A table of QName to XmlSerialize
		/// entries which give hints as to how the child is to be created.
		/// </param>
		/// <returns>A new IEntry representing the new child.</returns>
		IEntry createChild(XmlQualifiedName childType,
			string child, Hashtable creationHints);

		/// <summary>
		/// Similar to createChild, this adds a new entry to context
		/// space under this context.  However, this doesn't create
		/// the child but rather just links it in under a given name.
		/// Because of this, the type of the child doesn't have to even
		/// be known.
		/// </summary>
		/// <param name="type">If available, the QName of a type which
		/// this child should have.  It's permissable for this parameter
		/// to be null, or even completely arbitrary though if this
		/// type is not one of the well known ones, it will show up in
		/// context space as an unknown.</param>
		/// <param name="name">The name of the child we wish to create.
		/// </param>
		/// <param name="epr">The EPR to link into context space.  This
		/// parameter is required to be non-null.</param>
		/// <param name="hardLink">Specifies whether or not this link
		/// should be a hard link, or a soft link.  Currently, soft links
		/// are not supported.</param>
		/// <returns>The new IEntry instance which represents the new
		/// linked element.</returns>
		IEntry link(XmlQualifiedName type, string name, 
			EndpointReferenceType epr, bool hardLink);

		/// <remarks>
		/// Remove an entry from context space.  This method is currently
		/// unimplemented in the grid system.
		/// </remarks>
		IEntry remove(string child);

		/// <summary>
		/// Create a new sub-context for this context.
		/// </summary>
		/// <param name="child">The name the new sub-context should have
		/// in context space.</param>
		/// <param name="creationHints">Creation hints as
		/// per the <see cref="Grid.ContextSpace.IContext.createChild"/>
		/// method.</param>
		/// <returns>The new IEntry that represents the new context
		/// that was created.</returns>
		IEntry mkdir(string child, Hashtable creationHints);

		/// <summary>
		/// Create a new sub-file for this context or merely touch an
		/// existing one.
		/// </summary>
		/// <param name="child">The name the new or existing file</param>
		/// <param name="creationHints">Creation hints as
		/// per the <see cref="Grid.ContextSpace.IContext.createChild"/>
		/// method.</param>
		/// <returns>The new IEntry that represents the new or existing
		/// file</returns>
		IEntry touch(string child, Hashtable creationHints);
	}
}