// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;

namespace Grid.ContextSpace
{
	/// <summary>
	/// A comparer which is used by ls to sort entries alphabetically.
	/// </summary>
	public class IEntryComparer : IComparer
	{
		/// <summary>
		/// The public singleton instance of this comparer.
		/// </summary>
		static public IEntryComparer Instance = new IEntryComparer();

		#region IComparer Members

		/// <remarks><see cref="System.Collections.IComparer.Compare"/>
		/// </remarks>
		public int Compare(object x, object y)
		{
			IEntry one = (IEntry)x;
			IEntry two = (IEntry)y;

			return one.EntryName.CompareTo(two.EntryName);
		}

		#endregion
	}

	/// <summary>
	/// This is the common interface to ALL entries that can appear in
	/// context space.  An entry is a collection of information and
	/// funtionallity that entries in context space have.  It's important
	/// to realize that an entry is the resolution of a context space
	/// relative to a current position in context space.  Names, paths,
	/// etc. are relative to that, not absolute unless the path given
	/// to create this entry was absolute.  Further, entries which have
	/// multiple names in context space would technically have multiple
	/// enties that represented them.
	/// </summary>
	public interface IEntry
	{
		/// <summary>
		/// All entries in context space have a type, even if that type is
		/// not actually known.  This retrieves that type.
		/// </summary>
		XmlQualifiedName EntryType { get; }

		/// <summary>
		/// The entry name of the entry in context space.
		/// </summary>
		string EntryName { get; }

		/// <summary>
		/// Indicates whether this entry refers to an existing entry in
		/// context space, or one that has yet to be created (i.e., the
		/// path used to construct this entry does not resolve to an
		/// existing entry yet).  If this does not exist, then the
		/// IEntry should also implement the INotAvailableEntry interface.
		/// </summary>
		bool Exists { get; }

		/// <summary>
		/// Retrieves the parent of this entry (if available).  Be aware
		/// that knowing the parent is a bonus, not a guarantee.  You could
		/// very well get a null back for an entry that in reality does
		/// have parents.  The only time you are guaranteed to get your
		/// parent is when you are an entry that does not exist yet.
		/// </summary>
		IEntry Parent { get; }

		/// <summary>
		/// Indicates whether or not this entry is a grid entry or a local
		/// entry (on a local file system).  If you are not local, then you
		/// should implement, in addition to the IEntry interface, the
		/// IGridEntry interface.
		/// </summary>
		bool Local { get; }

		/// <summary>
		/// As far back as possible, retrieve this entries full path.  This
		/// method could return as little information as the current name, or
		/// as much as the full or absolute path to it from the current
		/// position.  In general, it will be the path given to create this
		/// entry.
		/// </summary>
		string Path { get; }

		/// <summary>
		/// All entries can be listed in an ls command or operation.  This
		/// method generates the information for that listing.
		/// </summary>
		/// <param name="isLong">Should we attempt to get long listing
		/// information.  This is mostly a hint that allows the target
		/// entry to decide whether or not getting long information is
		/// necessary so that it can skip that if getting the information
		/// is costly.</param>
		/// <returns>An LsFormat instance with the ls information to
		/// display.</returns>
		LsFormat format(
			bool isLong);
	}
}