// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An interface that all IEntries which refer to files must
	/// implement.
	/// </summary>
	public interface IFile
	{
		/// <summary>
		/// Retrieves the size (in bytes) of this file.
		/// </summary>
		long Size { get; }

		/// <summary>
		/// Read a block of data from the file.
		/// </summary>
		/// <param name="offset">The offset into the file at which
		/// to start reading.</param>
		/// <param name="length">The length of the block to read.</param>
		/// <returns>Returns the new block of data.  Note that this byte
		/// array could be shorter then the block request, or even empty
		/// if the end of the file has been reached.</returns>
		byte[] read(long offset, int length);

		/// <summary>
		/// Write a block of data to the file.
		/// </summary>
		/// <param name="offset">The offset into the file at which to 
		/// start writing.</param>
		/// <param name="data">The block of data to write.  This parameter
		/// cannot be null.</param>
		/// <returns>The number of bytes written.  Note that this value could
		/// in theory be 0 or smaller then the amount of data requested to
		/// be written.</returns>
		int write(long offset, byte []data);

		/// <summary>
		/// Append a bunch of data to the file while truncating it
		/// </summary>
		/// <param name="data">The data to write to the beginning of the file.
		/// </param>
		/// <returns>The number of bytes written.  Note that this value could
		/// in theory be 0 or smaller then the amount of data requested to
		/// be written.</returns>
		int truncAppend(byte []data);

		/// <summary>
		/// Append a bunch of data to the file.
		/// </summary>
		/// <param name="data">The data to write to the end of the file.
		/// </param>
		/// <returns>The number of bytes written.  Note that this value could
		/// in theory be 0 or smaller then the amount of data requested to
		/// be written.</returns>
		int append(byte []data);

		/// <summary>
		/// Open a new Stream for this file so that it can be read.
		/// </summary>
		/// <returns>A new stream which can be used to access the file.
		/// </returns>
		Stream OpenRead();

		/// <summary>
		/// Open a new Stream for this file so that it can be written.
		/// </summary>
		/// <returns>A new stream which can be used to modify the file.
		/// </returns>
		Stream OpenWrite();

		/// <summary>
		/// Open a new stream for this file which can be used to access
		/// it.
		/// </summary>
		/// <param name="mode">The mode under which this file is to be
		/// opened.  If none is given, ReadWrite is assumed.</param>
		/// <returns>A new stream which can be used to access the file.
		/// </returns>
		Stream Open(FileMode mode);

		/// <summary>
		/// Open a new stream for this file which can be used to access
		/// it.
		/// </summary>
		/// <param name="mode">The mode under which this file is to be
		/// opened.  If none is given, ReadWrite is assumed.</param>
		/// <param name="access">The type of access that is requested.</param>
		/// <returns>A new stream which can be used to access the file.
		/// </returns>
		Stream Open(FileMode mode, FileAccess access);
	}
}