// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.IO;
using System.Xml;

using UVa.GCG.WSRF.Common.WS.Addressing;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An IEntry type which represents a local directory on disk.
	/// </summary>
	public class LocalDirectory : AbstractLocalEntry, IContext
	{
		/// <summary>
		/// Construct a new LocalDirectory instance.
		/// </summary>
		/// <param name="parent">The parent entry for this local
		/// directory instance.  This parameter is allowed to be
		/// null if the parent is unknown or hasn't been created
		/// as an IEntry.</param>
		/// <param name="dInfo">The directory info that describes
		/// this directory on disk.</param>
		public LocalDirectory(IEntry parent, DirectoryInfo dInfo)
			: base(parent, dInfo)
		{
		}

		#region Private Members

		private IEntry createChildElement(FileSystemInfo info)
		{
			if (info is DirectoryInfo)
				return new LocalDirectory(this, (DirectoryInfo)info);
			else
				return new LocalFile(this, (FileInfo)info);
		}

		#endregion

		#region IContext Members

		/// <summary><see cref="IContext.Children"/></summary>
		public IEntry[] Children
		{
			get
			{
				FileSystemInfo []children = 
					((DirectoryInfo)_info).GetFileSystemInfos();
				IEntry []ret = new IEntry[children.Length];
				for (int lcv = 0; lcv < children.Length; lcv++)
					ret[lcv] = createChildElement(children[lcv]);

				return ret;
			}
		}

		/// <summary><see cref="IContext.this[string]"/></summary>
		public IEntry this[string child]
		{
			get
			{
				FileSystemInfo []children = 
					((DirectoryInfo)_info).GetFileSystemInfos();
				foreach (FileSystemInfo info in children)
				{
					if (info.Name.Equals(child))
						return createChildElement(info);
				}

				return null;
			}
		}

		/// <summary><see cref="IContext.createChild"/></summary>
		public IEntry createChild(XmlQualifiedName childType, 
			string child, Hashtable creationHints)
		{
			if (childType.Equals(WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
			{
				return new LocalDirectory(this,
					((DirectoryInfo)_info).CreateSubdirectory(child));
			} 
			else if (childType.Equals(WellknownEntryTypes.ENTRY_TYPE_FILE))
			{
				FileInfo file = new FileInfo(
					string.Format("{0}\\{1}", _info.FullName, child));
				if (file.Exists)
					throw new PathAlreadyExistsException(file.FullName);
				using (FileStream stream = file.Create())
				{
				}
				return new LocalFile(this, file);
			} 
			else
			{
				throw new UnimplementedException(
					this.GetType(),
					this.GetType().GetMethod("createChild",
					new Type[] {childType.GetType(), child.GetType(),
								   creationHints.GetType()}),
					"The ability to create children of a weird type "
					+ "under a local context is not yet supported.");
			}
		}

		/// <summary><see cref="IContext.link"/></summary>
		public IEntry link(XmlQualifiedName type, string name, 
			EndpointReferenceType epr, bool hardLink)
		{
			throw new UnimplementedException(
				this.GetType(),
				this.GetType().GetMethod("link",
				new Type[] {type.GetType(), name.GetType(),
							   epr.GetType(), hardLink.GetType()}),
				"The ability to create children of a weird type "
				+ "under a local context is not yet supported.");
		}

		/// <summary><see cref="IContext.remove"/></summary>
		public IEntry remove(string child)
		{
			throw new UnimplementedException(
				this.GetType(), this.GetType().GetMethod("remove"));
		}

		/// <summary><see cref="IContext.mkdir"/></summary>
		public IEntry mkdir(string child, Hashtable creationHints)
		{
			return createChild(
				WellknownEntryTypes.ENTRY_TYPE_CONTEXT, child, creationHints);
		}

		/// <summary><see cref="IContext.touch"/></summary>
		public IEntry touch(string child, Hashtable creationHints)
		{
			return createChild(
				WellknownEntryTypes.ENTRY_TYPE_FILE, child, creationHints);
		}

		#endregion

		#region IEntry Members

		/// <summary><see cref="IEntry.format"/></summary>
		public override LsFormat format(bool isLong)
		{
			return new LsFormat(EntryName, "[LOCAL-DIR]");
		}

		#endregion
	}
}