// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;

using UVa.GCG.WSRF.Common.WS.Addressing;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An IEntryFactory type which creates IEntries that are
	/// either LocalDirectory or LocalFile entries.
	/// </summary>
	public class LocalEntryFactory : IEntryFactory
	{
		static private LocalEntryFactory _instance = new LocalEntryFactory();

		/// <summary>
		/// Retrieve the singleton LocalEntryFactory instance.
		/// </summary>
		static public LocalEntryFactory Instance
		{
			get
			{
				return _instance;
			}
		}

		private LocalEntryFactory()
		{
		}

		#region IEntryFactory Members

		/// <summary><see cref="IEntryFactory.CommonName"/></summary>
		public string CommonName
		{
			get
			{
				return "Local Entry";
			}
		}

		/// <summary><see cref="IEntryFactory.EntryType"/></summary>
		public XmlQualifiedName EntryType
		{
			get
			{
				return null;
			}
		}

		/// <summary><see cref="IEntryFactory.create"/></summary>
		public IEntry create(string path, EndpointReferenceType epr, 
			IEntry parent)
		{
			// If the path wasn't given, the let's assume that they meant
			// simply the current directory.
			if (path.Length == 0)
				return new LocalDirectory(parent, new DirectoryInfo("."));

			// If it exists as a file, then it's a local file.
			if (File.Exists(path))
				return new LocalFile(parent, new FileInfo(path));
			// If it exists as a directory, then it's a local directory.
			if (Directory.Exists(path))
				return new LocalDirectory(parent, new DirectoryInfo(path));

			// Otherwise, it doesn't exist at all.
			return new LocalNotAvailableEntry(
				create(Path.GetDirectoryName(path), null, null), 
				Path.GetFileName(path));
		}

		#endregion
	}
}