// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;

namespace Grid.ContextSpace
{
	/// <summary>
	/// This is an entry type that represents a local file on disk.
	/// </summary>
	public class LocalFile : AbstractLocalEntry, IFile
	{
		/// <summary>
		/// Construct a new LocalFile.
		/// </summary>
		/// <param name="parent">The parent entry (if known).  Note
		/// that this could be null if the parent isn't known or
		/// hasn't been looked up.</param>
		/// <param name="fInfo">The FileInfo that describes this local
		/// file.</param>
		public LocalFile(IEntry parent, FileInfo fInfo)
			: base(parent, fInfo)
		{
		}

		#region IFile Members

		/// <remarks><see cref="IFile.Size"/></remarks>
		public long Size
		{
			get
			{
				return ((FileInfo)_info).Length;
			}
		}

		/// <remarks><see cref="IFile.read"/></remarks>
		public byte[] read(long offset, int length)
		{
			using (FileStream stream = 
				((FileInfo)_info).Open(FileMode.Open, FileAccess.Read,
					   FileShare.None))
			{
				stream.Seek(offset, SeekOrigin.Begin);
				byte []data = new byte[length];
				int bRead = stream.Read(data, 0, length);
				if (bRead != length)
				{
					byte []ret = new byte[bRead];
					Array.Copy(data, 0, ret, 0, bRead);
					return ret;
				}

				return data;
			}
		}

		/// <remarks><see cref="IFile.write"/></remarks>
		public int write(long offset, byte[] data)
		{
			using (FileStream stream = 
				((FileInfo)_info).Open(FileMode.Open, FileAccess.Write,
					   FileShare.None))
			{
				stream.Seek(offset, SeekOrigin.Begin);
				stream.Write(data, 0, data.Length);
				return data.Length;
			}
		}

		/// <remarks><see cref="IFile.truncAppend"/></remarks>
		public int truncAppend(byte[] data)
		{
			using (FileStream stream =
				((FileInfo)_info).Open(FileMode.Truncate,
					   FileAccess.Write, FileShare.None))
			{
				stream.Write(data, 0, data.Length);
				return data.Length;
			}
		}

		/// <remarks><see cref="IFile.append"/></remarks>
		public int append(byte []data)
		{
			using (FileStream stream = ((FileInfo)_info).Open(
					   FileMode.Append, FileAccess.Write,
					   FileShare.None))
			{
				stream.Write(data, 0, data.Length);
				return data.Length;
			}
		}

		/// <remarks><see cref="IFile.OpenRead"/></remarks>
		public Stream OpenRead()
		{
			return ((FileInfo)_info).OpenRead();
		}

		/// <remarks><see cref="IFile.OpenWrite"/></remarks>
		public Stream OpenWrite()
		{
			return ((FileInfo)_info).OpenWrite();
		}

		/// <remarks><see cref="IFile.Open"/></remarks>
		public Stream Open(FileMode mode)
		{
			return ((FileInfo)_info).Open(mode);
		}

		/// <remarks><see cref="IFile.Open"/></remarks>
		public Stream Open(FileMode mode, FileAccess access)
		{
			return ((FileInfo)_info).Open(mode, access);
		}

		#endregion

		#region IEntry Members

		/// <remarks><see cref="IEntry.format"/></remarks>
		public override LsFormat format(bool isLong)
		{
			if (isLong)
				return new LsFormat(EntryName,
					string.Format("[LOCAL-FILE:  {0} bytes]",
						Size));
			else
				return new LsFormat(EntryName);
		}

		#endregion
	}
}