// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.IO;
using System.Xml;

namespace Grid.ContextSpace
{
	/// <summary>
	/// An IEntry type that represents a path that is known to be in the local
	/// file space, but doesn't yet exist.
	/// </summary>
	public class LocalNotAvailableEntry : INotAvailableEntry
	{
		private IEntry _parent;
		private string _name;

		/// <summary>
		/// Construct a new LocalNotAvailableEntry instance.
		/// </summary>
		/// <param name="parent">The parent entry for this IEntry.  Unlike
		/// most IEntry instances, for LocalNotAvailableEntry instanecs, the
		/// parent MUST be non-null.</param>
		/// <param name="name">The name in the local space that this entry
		/// has.</param>
		public LocalNotAvailableEntry(IEntry parent,
			string name)
		{
			if (parent == null)
				throw new ArgumentNullException("parent",
					"The \"parent\" parameter cannot be null.");

			_parent = parent;
			_name = name;
		}

		#region INotAvailableEntry Members

		/// <remarks><see cref="INotAvailableEntry.createDirectory"/></remarks>
		public IEntry createDirectory(bool createParents, Hashtable creationHints)
		{
			IEntry parent = _parent;
			// Does the parent exist?
			if (!parent.Exists)
			{
				// If not we must create the parent if we are allowed to do
				// so.  If not, throw an exception.
				if (!createParents)
					throw new NoSuchPathException(
						parent.Path);
				INotAvailableEntry p = (INotAvailableEntry)parent;
				parent = p.createDirectory(createParents, creationHints);
			}

			IContext c = (IContext)parent;
			return c.mkdir(_name, creationHints);
		}

		/// <remarks><see cref="INotAvailableEntry.createFile"/></remarks>
		public IEntry createFile(bool createParents, Hashtable creationHints)
		{
			IEntry parent = _parent;
			// Does the parent exist?
			if (!parent.Exists)
			{
				// If not we must create the parent if we are allowed to do
				// so.  If not, throw an exception.
				if (!createParents)
					throw new NoSuchPathException(
						parent.Path);
				INotAvailableEntry p = (INotAvailableEntry)parent;
				parent = p.createDirectory(createParents, creationHints);
			}

			IContext c = (IContext)parent;
			return c.touch(_name, creationHints);
		}

		/// <remarks><see cref="INotAvailableEntry.create"/></remarks>
		public IEntry create(bool createParents, XmlQualifiedName type,
			Hashtable creationHints)
		{
			if (type.Equals(WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
				return createDirectory(createParents, creationHints);
			if (type.Equals(WellknownEntryTypes.ENTRY_TYPE_FILE))
				return createFile(createParents, creationHints);

			throw new ContextSpaceException(string.Format(
				"Not allowed to create a local entry of type \"{0}\".",
				type));
		}

		#endregion

		#region IEntry Members

		/// <remarks><see cref="IEntry.EntryType"/></remarks>
		public XmlQualifiedName EntryType
		{
			get
			{
				return null;
			}
		}

		/// <remarks><see cref="IEntry.EntryName"/></remarks>
		public string EntryName
		{
			get
			{
				return _name;
			}
		}

		/// <remarks><see cref="IEntry.Exists"/></remarks>
		public bool Exists
		{
			get
			{
				return false;
			}
		}

		/// <remarks><see cref="IEntry.Parent"/></remarks>
		public IEntry Parent
		{
			get
			{
				return _parent;
			}
		}

		/// <remarks><see cref="IEntry.Path"/></remarks>
		public string Path
		{
			get
			{
				string parentPath = _parent.Path;
				char lastChar = parentPath[
					parentPath.Length - 1];
				if (lastChar == '/' || lastChar == '\\')
					return string.Format("{0}{1}", parentPath, _name);
				else
					return string.Format("{0}\\{1}", parentPath, _name);
			}
		}

		/// <remarks><see cref="IEntry.Local"/></remarks>
		public bool Local
		{
			get
			{
				return true;
			}
		}

		/// <remarks><see cref="IEntry.format"/></remarks>
		public LsFormat format(bool isLong)
		{
			// Should never be called.
			return null;
		}

		#endregion
	}
}