// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;

using Grid;
using Grid.Env;

namespace Grid.ContextSpace
{
	/// <summary>
	/// Represents a collection of paths that can be resolved to a collection
	/// of IEntries.  This class is largely superfluous now and the
	/// <see cref="ContextUtilities"/> class should instead be used.  This class
	/// will be removed or trimmed soon to reflect this.
	/// </summary>
	public class PathExpressionCollection
	{
		#region Constants

		private const string _LOCAL_HEADER = "local:";
		private const string _GRID_HEADER = "grid:";

		#endregion

		private bool _containsWildcards = false;

		/// <summary>
		/// List of IEntry
		/// </summary>
		private ArrayList _resolved = new ArrayList();

		/// <summary>
		/// List of string
		/// </summary>
		private ArrayList _expressions = new ArrayList();

		/// <summary>
		/// Construct a new PathExpressionCollection
		/// </summary>
		/// <param name="pathExpressions">A list of path expressions
		/// which represent local or grid paths that can contain
		/// wildcards.</param>
		public PathExpressionCollection(string []pathExpressions)
		{
			foreach (string expr in pathExpressions)
				add(expr);
		}

		/// <summary>
		/// Construct a new PathExpressionCollection
		/// </summary>
		/// <param name="pathExpression">A path expression
		/// which represents a local or a grid path that can contain
		/// wildcards.</param>
		public PathExpressionCollection(string pathExpression)
		{
			add(pathExpression);
		}

		/// <summary>
		/// Construct an empty PathExpressionCollection.
		/// </summary>
		public PathExpressionCollection()
		{
		}

		/// <summary>
		/// Add a new path expression to this collection.
		/// </summary>
		/// <param name="expression">The new path expression to add.
		/// This path expression can be local or remote, and can contain
		/// wildcards.</param>
		public void add(string expression)
		{
			_expressions.Add(expression);

			if (Wildcards.hasWildcards(expression))
				_containsWildcards = true;
		}

		/// <summary>
		/// Indicates whether or not their are any wildcards currently
		/// in the path expressions.  This could be used to indicate the
		/// amount of effort that might go into figuring out the count.
		/// </summary>
		public bool ContainsWildcards
		{
			get
			{
				return _containsWildcards;
			}
		}

		/// <summary>
		/// How many paths are represented here?  NOTE that resolving
		/// this property could be expensive the first time involving
		/// Context space lookups if wild card expansion is required.
		/// </summary>
		public int Count
		{
			get
			{
				// If there aren't currently any wildcards, then we
				// can count the entries we have and that is it.
				if (!_containsWildcards)
					return (_resolved.Count + _expressions.Count);

				// Otherwise, we have to resolve all paths in order to
				// count.
				internalResolveAll(false);
				return _resolved.Count;
			}
		}

		/// <summary>
		/// Lookup the current path expressions and force that result to be
		/// a singleton.  If more then one path is resultant, then throw an
		/// exception.
		/// </summary>
		/// <returns>The IEntry that matches the given path.</returns>
		public IEntry resolveSingleton()
		{
			internalResolveAll(true);
			if (_resolved.Count != 1)
				throw new SingletonFailureException();

			return (IEntry)_resolved[0];
		}

		/// <summary>
		/// Lookup all IEntries that match all given path expressions including
		/// wild card expansion.
		/// </summary>
		/// <returns>A list of IEntry results</returns>
		public IEntry[] resolveAll()
		{
			internalResolveAll(false);
			IEntry []ret = new IEntry[_resolved.Count];
			_resolved.CopyTo(ret);

			return ret;
		}

		private void internalResolveAll(bool failAfterOne)
		{
			// The fail after one parameter is an optimization that
			// allows us to skip resolving a bunch of entries if the
			// user wants to throw an exception for more the one
			// entry resulting from a resolution.
			if (failAfterOne && _resolved.Count > 1)
				return;

			// We clone this list because we are about to go through
			// the list and remove elements as we go.
			ArrayList copy = (ArrayList)_expressions.Clone();
			foreach (string pathExpr in copy)
			{
				// Resolve this single path.  NOTE that this could
				// create multiple entries (due to wildcards).
				resolvePath(pathExpr);
				_expressions.Remove(pathExpr);

				// Let's see if we can fail early.
				if (failAfterOne && _resolved.Count > 1)
					return;
			}

			// Once we are done resolving all the entries, then we
			// know that there can't be any more wildcards.
			_containsWildcards = false;
		}

		private void resolvePath(string pathExpr)
		{
			bool allowNonExistant = !Wildcards.hasWildcards(pathExpr);

			// First, get an entry for the root of the file system
			// from which we will start searching (in otherwords, an
			// IEntry that represents the first component of the path).
			IEntry pathRoot = findRoot(ref pathExpr);

			// Now that we have a root from which to start, resolve the
			// rest of the path.
			resolvePath(pathRoot, pathExpr, allowNonExistant);
		}

		private IEntry findRoot(ref string pathExpr)
		{
			// If the path expression has the local: header, then
			// the path represents a file or path in the local file system.
			if (pathExpr.StartsWith(_LOCAL_HEADER))
			{
				// Local entries are easier because .NET already knows how
				// to look them up and get their parents, so we will simply
				// create an entry that recognizes this fact.
				IEntry entry = LocalEntryFactory.Instance.create(
					pathExpr.Substring(_LOCAL_HEADER.Length), null, null);
				pathExpr = null;
				return entry;
			}

			// We now know it's a grid entry, but they may have specified
			// that for clarity, so we check to strip out the header if
			// necessary.
			if (pathExpr.StartsWith(_GRID_HEADER))
				pathExpr = pathExpr.Substring(_GRID_HEADER.Length);

			// If the path expression equals /, then we know that
			// the root IEntry is in fact the root of context space.
			// Further, we set the pathExpr equal to null so that
			// we don't need to do any more look ups.
			if (pathExpr.Equals("/"))
			{
				pathExpr = null;
				return new GridContext(
					GridLibEnvironment.Instance.ContextEnvironment.RootContext,
					"/", null);
			}

			// If on the other hand, the expression equals ., then we also
			// know the full IEntry for the search, so clear out the
			// expresion, and return the current context.
			if (pathExpr.Equals("."))
			{
				pathExpr = null;
				return new GridContext(
					GridLibEnvironment.Instance.ContextEnvironment.CurrentContext,
					".", null);
			}

			// If the pathexpression starts with a / (but isn't == to /), then
			// the root of this search will be the roof of context space, and
			// we can strip off that leading / from the expression for the
			// remainder of the search.
			if (pathExpr.StartsWith("/"))
			{
				pathExpr = pathExpr.Substring(1);
				return new GridContext(
					GridLibEnvironment.Instance.ContextEnvironment.RootContext,
					"/", null);
			} 
				// If on the other hand, the expression starts with a dot-slash, 
				// then we know that the expression starts from current.  Further,
				// since we know this, we have to strip that ./ off of the path.
			else if (pathExpr.StartsWith("./"))
			{
				pathExpr = pathExpr.Substring(2);
				return new GridContext(
					GridLibEnvironment.Instance.ContextEnvironment.CurrentContext,
					".", null);
			} else
				// Finally, the path could just start with a child element of
				// the current context, so set the current, and get on with
				// the searching.
			{
				return new GridContext(
					GridLibEnvironment.Instance.ContextEnvironment.CurrentContext,
					".", null);
			}
		}

		private void resolvePath(IEntry parent, string pathExpr,
			bool allowNonExistant)
		{
			// If the remaining path expression is null or empty, then
			// we've done all the looking up we need in order to resolve
			// this entry.
			if (pathExpr == null || pathExpr.Length == 0)
			{
				_resolved.Add(parent);
				return;
			}

			// If we have gotten this far, then we have a parent who isn't a
			// context (and therefor can't have children), but we have path
			// expression remaining to resolve.  This isn't an error necessarily
			// as it could be the result of over-aggressive wildcard expansion, so
			// we simply don't add it.
			if (!parent.EntryType.Equals(WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
				return;

			// Find the next / so we can further break the path into it's next
			// components.
			string next;
			int index = pathExpr.IndexOf('/');
			if (index < 0)
			{
				// Looks like we found the last expression element.
				next = pathExpr;
				pathExpr = null;
			} 
			else
			{
				// We get the next element in the path expression, and
				// trim it off the remainder.
				next = pathExpr.Substring(0, index);
				pathExpr = pathExpr.Substring(index + 1);
			}

			// We have a pattern (which could contain wildcards) and
			// a parent, so look up the children and get all the ones
			// that match the pattern.
			IEntry[] children = match((IContext)parent, next);
			if (children.Length == 0 && allowNonExistant)
			{
				children = new IEntry[]
					{
						createNonExistant(parent, next, pathExpr)
					};
				pathExpr = null;
			}
			
			foreach (IEntry child in children)
				resolvePath(child, pathExpr, allowNonExistant);
		}

		private IEntry[] match(IContext parent, string pattern)
		{
			ArrayList tmp = new ArrayList();
			IEntry []allChildren = parent.Children;
			foreach (IEntry child in allChildren)
			{
				if (Wildcards.matches(child.EntryName, pattern))
					tmp.Add(child);
			}

			IEntry []ret = new IEntry[tmp.Count];
			tmp.CopyTo(ret);

			return ret;
		}

		private IEntry createNonExistant(IEntry parent, string name, string rest)
		{
			while (name != null && name.Length > 0)
			{
				parent = new GridNotAvailableEntry(name, parent);
				if (rest == null || rest.Length == 0)
					break;
				int index = rest.IndexOf('/');
				if (index < 0)
				{
					name = rest;
					rest = null;
				} 
				else
				{
					name = rest.Substring(0, index);
					rest = rest.Substring(index + 1);
				}
			}

			return parent;
		}
	}
}