// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.WS;
using UVa.GCG.WSRF.Common.WS.Addressing;

using Grid;

namespace Grid.Env
{
	/// <summary>
	/// Instances of this class represent the known context state for
	/// a given session.  This state indicates the root and current context
	/// for that session.
	/// </summary>
	[XmlRoot(GridConstants.CONTEXT_ENVIRONMENT_ROOT,
		 Namespace=GridConstants.GRID_NS)]
	public class CurrentContextEnvironment
	{
		#region Private Members

		private bool _modified = false;

		private EndpointReferenceType _rootContext;
		private EndpointReferenceType _currentContext;

		#endregion

		#region Public Members

		/// <summary>
		/// Get or set the root context for this session.
		/// </summary>
		public EndpointReferenceType RootContext
		{
			get
			{
				if (_rootContext == null)
					throw new EnvironmentUnsetException();

				return _rootContext;
			}

			set
			{
				_modified = true;
				_rootContext = value;
			}
		}

		/// <summary>
		/// Get or set the Current context for this session.
		/// </summary>
		public EndpointReferenceType CurrentContext
		{
			get
			{
				if (_currentContext == null)
					throw new EnvironmentUnsetException();

				return _currentContext;
			}

			set
			{
				_modified = true;
				_currentContext = value;
			}
		}

		/// <summary>
		/// Get or set a flag indicating whether or not the state has
		/// been modified since it was last read in.  This flag is a
		/// convenience item and is NOT automatically handled by instances
		/// of this class.  In fact, it isn't even used by this class.  In
		/// the grid lib, it is used by the GridLibEnvironment class.
		/// </summary>
		[XmlIgnore]
		public bool Modified
		{
			get
			{
				return _modified;
			}

			set
			{
				_modified = value;
			}
		}

		/// <summary>
		/// Construct a new CurrentContextEnvironment instance with null
		/// context entries.
		/// </summary>
		public CurrentContextEnvironment()
		{
			_rootContext = null;
			_currentContext = null;
		}

		#endregion

		#region Static Methods

		/// <summary>
		/// Store the given CurrentContextEnvironment in the appropriate
		/// session file.
		/// </summary>
		/// <param name="env">The environment to store.</param>
		static public void store(CurrentContextEnvironment env)
		{
			store(env, FullPath);
			env._modified = false;
		}

		/// <summary>
		/// Store the given context environment in the specified file.
		/// </summary>
		/// <param name="env">The context environment to store.</param>
		/// <param name="filename">The file name in which to store the
		/// context environment.</param>
		static public void store(CurrentContextEnvironment env, string filename)
		{
			using (StreamWriter writer = new StreamWriter(filename))
			{
				XmlSerializer ser = new XmlSerializer(typeof(CurrentContextEnvironment));
				ser.Serialize(writer, env);
			}

			env._modified = false;
		}

		/// <summary>
		/// Load a context environment from the wellknown session file.
		/// </summary>
		/// <returns>The newly loaded context environment.</returns>
		static public CurrentContextEnvironment load()
		{
			try
			{
				return load(FullPath);
			}
			catch (FileNotFoundException fnfe)
			{
				throw new DisconnectedException(fnfe);
			}
		}

		/// <summary>
		/// Load a context environment from the given session file.
		/// </summary>
		/// <param name="path">The path to a file from which we want
		/// to load a context environment.</param>
		/// <returns>The newly loaded context environment.</returns>
		static public CurrentContextEnvironment load(string path)
		{
			using (StreamReader reader = new StreamReader(path))
			{
				XmlSerializer ser = new XmlSerializer(
					typeof(CurrentContextEnvironment));
				return (CurrentContextEnvironment)ser.Deserialize(reader);
			}
		}

		/// <summary>
		/// Retrieve the full path on local disk of the wellknown
		/// session file.
		/// </summary>
		static public string FullPath
		{
			get
			{
				return string.Format("{0}/.grid-data",
					System.Environment.GetFolderPath(
					System.Environment.SpecialFolder.LocalApplicationData));	
			}
		}

		#endregion
	}
}