// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;

using Grid.Output;

namespace Grid.Env
{
	/// <summary>
	/// This is a singleton class which represents all of the current state for
	/// a GridLib environment.  For the most part, this is current context and
	/// root context but will also contain caches, etc.
	/// </summary>
	public class GridLibEnvironment : IDisposable
	{
		/// <summary>
		/// Destruct the given GridLibEnvironment instance.
		/// </summary>
		~GridLibEnvironment()
		{
			Dispose();
		}

		#region Statics

		static private GridLibEnvironment _instance = new GridLibEnvironment();

		/// <summary>
		/// Retrieve the singleton instance.
		/// </summary>
		static public GridLibEnvironment Instance
		{
			get
			{
				return _instance;
			}
		}

		#endregion

		#region Members

		private OutputConsole _outConsole = new OutputConsole();
		private CurrentContextEnvironment _contextEnv = null;
		private bool _alreadyDisposed = false;

		private bool _modified = false;

		#endregion

		#region Properties

		/// <summary>
		/// Get the output console for this session.  This console is used to
		/// keep track of where all kinds of output shoudl go (including
		/// /dev/null).
		/// </summary>
		public OutputConsole Console
		{
			get
			{
				return _outConsole;
			}
		}

		/// <summary>
		/// Retrieve or set the current context environment.  While the compiler
		/// won't keep you from setting the Current and Root contexts from this
		/// environment, you SHOULD NOT do so.  Rather, create a new one and
		/// set it's members in the constructor, then set this property with the
		/// new environment.  This will correctly update the modified flag and
		/// cause the new environment to be written to disk as appropriate.
		/// </summary>
		public CurrentContextEnvironment ContextEnvironment
		{
			get
			{
				if (_contextEnv == null)
					_contextEnv = CurrentContextEnvironment.load();

				return _contextEnv;
			}

			set
			{
				_modified = true;
				_contextEnv = value;
			}
		}

		/// <summary>
		/// Get or set a flag indicating whether or not the environment
		/// has been modified.
		/// </summary>
		public bool Modified
		{
			get
			{
				return _modified;
			}

			set
			{
				_modified = value;
			}
		}

		#endregion

		#region IDisposable Members

		/// <remarks><see cref="System.IDisposable.Dispose"/></remarks>
		public void Dispose()
		{
			if (!_alreadyDisposed)
			{
				_alreadyDisposed = true;

				if (_modified)
				{
					CurrentContextEnvironment.store(_contextEnv);
					_modified = false;
				}

				GC.SuppressFinalize(this);
			}
		}

		#endregion
	}
}