using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Serialization;

using Grid;
using Grid.Extensions;
using Grid.Env;
using Grid.ContextSpace;

namespace Grid.Extensions
{
	/// <summary>
	/// Collection of extension library descriptions.
	/// </summary>
	public class ExtensionCollection
	{
		private const int _BUFFER_SIZE = 1024 * 4;
		static private ExtensionCollection _instance = null;
		
		/// <summary>
		/// The singleton instance of this class.
		/// </summary>
		static public ExtensionCollection Instance
		{
			get
			{
				lock(typeof(ExtensionCollection))
				{
					if (_instance == null)
					{
						_instance = loadConfig();
					}

					return _instance;
				}
			}
		}

		private ArrayList _descriptions = null;

		private ExtensionCollection(ExtensionLibraryDescription []descriptions)
		{
			_descriptions = new ArrayList(descriptions);
		}

		private ExtensionCollection()
		{
			_descriptions = new ArrayList();
		}

		/// <remarks>
		/// <see cref="Grid.Operations.GridLibConfOperation.addExtensionLibrary"/>
		/// </remarks>
		public void addExtensionLibrary(
			string assemblyFile, string libraryName, string libraryDescription)
		{
			if (libraryDescription != null && libraryDescription.Length == 0)
				libraryDescription = null;

			FileInfo file = new FileInfo(assemblyFile);
			if (!file.Exists)
				throw new NoSuchPathException(assemblyFile);

			if (libraryName == null || libraryName.Length == 0)
				libraryName = getLibraryName(file.Name);

			lock (this)
			{
				foreach (ExtensionLibraryDescription desc in _descriptions)
				{
					if (desc.Name.Equals(libraryName))
						throw new ExtensionException(string.Format(
							"Extension library with name {0} already exists.", 
							libraryName));
				}

				Assembly assem = ExtensionLibraryDescription.loadAssembly(
					file);

				string targetFileName = string.Format("{0}.{1}",
					Guid.NewGuid().ToString(), file.Extension);
				ExtensionLibraryDescription description =
					new ExtensionLibraryDescription(libraryName, targetFileName,
					libraryDescription);
				copyFile(file, description.AssemblyFile);

				_descriptions.Add(description);
				storeConfig(this);
			}
		}

		/// <remarks>
		/// <see cref="Grid.Operations.GridLibConfOperation.addExtensionLibrary"/>
		/// </remarks>
		public void addExtensionLibrary(
			string assemblyFile, string libraryName)
		{
			addExtensionLibrary(assemblyFile, libraryName, null);
		}

		/// <remarks>
		/// <see cref="Grid.Operations.GridLibConfOperation.addExtensionLibrary"/>
		/// </remarks>
		public void addExtensionLibrary(
			string assemblyFile)
		{
			addExtensionLibrary(assemblyFile, null, null);
		}

		/// <remarks>
		/// <see cref="Grid.Operations.GridLibConfOperation.removeExtensionLibrary"/>
		/// </remarks>
		public void removeExtensionLibrary(string extensionLibraryName)
		{
			ExtensionLibraryDescription removee = null;

			lock (this)
			{
				foreach (ExtensionLibraryDescription desc in _descriptions)
				{
					if (desc.Name.Equals(extensionLibraryName))
					{
						removee = desc;
						break;
					}
				}

				if (removee != null)
				{	
					remove(removee);
					storeConfig(this);
				} 
				else
				{
					throw new ExtensionException(
						string.Format("Extension with name \"{0}\" not found.", 
						extensionLibraryName));
				}
			}
		}

		/// <remarks>
		/// <see cref="Grid.Operations.GridLibConfOperation.InstalledExtensionLibraries"/>
		/// </remarks>
		public ExtensionLibraryDescription[] InstalledExtensionLibraries
		{
			get
			{
				ExtensionLibraryDescription[] descs = 
					new ExtensionLibraryDescription[_descriptions.Count];
				_descriptions.CopyTo(descs);

				return descs;
			}
		}

		private void remove(ExtensionLibraryDescription desc)
		{
			// Can't delete this file because it is loaded.  We'll need to delete it lazily on
			// the next invocation.  Let's take care of that at another date.
//			desc.AssemblyFile.Delete();
			_descriptions.Remove(desc);
		}

		static private void copyFile(FileInfo source, FileInfo target)
		{
			using (FileStream sourceStream = source.OpenRead())
			{
				using (FileStream targetStream = target.OpenWrite())
				{
					byte []buffer = new byte[_BUFFER_SIZE];
					int bytesRead;

					while ( (bytesRead = sourceStream.Read(
						buffer, 0, buffer.Length)) > 0)
					{
						targetStream.Write(buffer, 0, bytesRead);
					}
				}
			}
		}

		static private XmlRootAttribute RootAttr
		{
			get
			{
				XmlRootAttribute attr = new XmlRootAttribute(
					GridConstants.GRID_LIB_CONF_EXT_LIB_ROOT);
				attr.Namespace = GridConstants.GRID_NS;

				return attr;
			}
		}

		static private void storeConfig(ExtensionCollection collection)
		{
			lock (collection)
			{
				FileInfo config = ExtensionLibraryDescription.Configuration;
				XmlSerializer ser = new XmlSerializer(
					typeof(ExtensionLibraryDescription[]), RootAttr);
				using (FileStream stream = config.Open(
						   FileMode.Create, 
						   FileAccess.Write, FileShare.None))
				{
					ser.Serialize(stream, collection.InstalledExtensionLibraries);
				}
			}
		}

		static private ExtensionCollection loadConfig()
		{
			try
			{
				FileInfo config = ExtensionLibraryDescription.Configuration;
				XmlSerializer ser = new XmlSerializer(
					typeof(ExtensionLibraryDescription[]), RootAttr);
				using (FileStream stream = config.Open(FileMode.Open, 
						   FileAccess.Read, FileShare.None))
				{
					return new ExtensionCollection(
						(ExtensionLibraryDescription[])ser.Deserialize(stream));
				}
			}
			catch (FileNotFoundException)
			{
				return new ExtensionCollection();
			}
		}

		static private string getLibraryName(string fileName)
		{
			int index = fileName.LastIndexOf('/');
			if (index >= 0)
				fileName = fileName.Substring(index + 1);

			index = fileName.LastIndexOf('.');
			if (index > 0)
				fileName = fileName.Substring(0, index);

			return fileName;
		}
	}
}