// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Serialization;

using Grid;
using Grid.ContextSpace;
using Grid.Commands;

namespace Grid.Extensions
{
	/// <summary>
	/// A class which contains information about a single extension library.
	/// </summary>
	[XmlRoot(GridConstants.GRID_LIB_CONF_EXT_LIB_DESC_ROOT,
		 Namespace=GridConstants.GRID_NS)]
	public class ExtensionLibraryDescription
	{
		#region Constants

		private const string _CONFIG_FILE_NAME = "extensions.config";

		#endregion

		private string _name;
		private string _description = null;
		private string _filename;

		/// <summary>
		/// Construct a new ExtensionLibraryDescription.
		/// </summary>
		/// <param name="name">The name the extension library should have.
		/// This parameter is NOT allowed to be null.</param>
		/// <param name="filename">The filename for the extension library.
		/// This parameter needs to be a relative file name with no path.
		/// </param>
		/// <param name="description">The description (if any) for this
		/// extension library.</param>
		public ExtensionLibraryDescription(string name,
			string filename, string description)
		{
			if (name == null)
				throw new ArgumentNullException("name",
					"Name parameter cannot be null.");

			if (filename == null)
				throw new ArgumentNullException("filename",
					"Filename parameter cannot be null.");

			if ((filename.IndexOf('/') >= 0) ||
				(filename.IndexOf('\\') >= 0))
				throw new ArgumentException(
					"filename parameter must be a relative filename with no path.",
					"filename");

			_name = name;
			_description = description;
			_filename = filename;
		}

		/// <summary>
		/// Construct a new ExtensionLibraryDescription.
		/// </summary>
		/// <param name="name">The name the extension library should have.
		/// This parameter is NOT allowed to be null.</param>
		/// <param name="filename">The filename for the extension library.
		/// This parameter needs to be a relative file name with no path.
		/// </param>
		public ExtensionLibraryDescription(string name,
			string filename)
			: this(name, filename, null)
		{
		}

		/// <summary>
		/// Default constructor for Xml Serialization
		/// </summary>
		public ExtensionLibraryDescription()
		{
			// This is ONLY for Xml Serialization
		}

		/// <summary>
		/// Get or set the Name for this ExtensionLibraryDescription instance.
		/// </summary>
		[XmlAttribute(GridConstants.GRID_LIB_CONF_EXT_LIB_DESC_NAME,
			 Namespace=GridConstants.GRID_NS)]
		public string Name
		{
			get
			{
				return _name;
			}

			set
			{
				_name = value;
			}
		}

		/// <summary>
		/// Get or set the Description for this ExtensionLibraryDescription instance.
		/// </summary>
		[XmlText]
		public string Description
		{
			get
			{
				return _description;
			}

			set
			{
				_description = value;
			}
		}

		/// <summary>
		/// Get or set the Filename for this ExtensionLibraryDescription instance.
		/// </summary>
		[XmlAttribute(GridConstants.GRID_LIB_CONF_EXT_LIB_DESC_FILE,
			 Namespace=GridConstants.GRID_NS)]
		public string Filename
		{
			get
			{
				return _filename;
			}

			set
			{
				_filename = value;
			}
		}

		private Assembly _assembly = null;

		/// <summary>
		/// Retrieve the loaded assembly for this library.
		/// </summary>
		[XmlIgnore]
		public Assembly Assembly
		{
			get
			{
				lock (this)
				{
					if (_assembly == null)
						_assembly = loadAssembly(
							AssemblyFile);

					return _assembly;
				}
			}
		}

		private IEntryFactory[] _entryFactories = null;

		/// <summary>
		/// Retrieve teh IEntryFactory singletons for this library.
		/// </summary>
		[XmlIgnore]
		public IEntryFactory[] EntryFactories
		{
			get
			{
				lock (this)
				{
					if (_entryFactories == null)
						_entryFactories = loadEntryFactories(Assembly);

					return _entryFactories;
				}
			}
		}

		private ICommandFactory[] _commandFactories = null;

		/// <summary>
		/// Retrieve the ICommandFactory singletons for this library.
		/// </summary>
		[XmlIgnore]
		public ICommandFactory[] CommandFactories
		{
			get
			{
				lock (this)
				{
					if (_commandFactories == null)
						_commandFactories = loadCommandFactories(Assembly);

					return _commandFactories;
				}
			}
		}

		static private DirectoryInfo _extensionDirectory = null;

		/// <summary>
		/// Retrieve the DirectoryInfo for the directory where extensions
		/// exist.
		/// </summary>
		[XmlIgnore]
		static public DirectoryInfo ExtensionDirectory
		{
			get
			{
				lock (typeof(ExtensionLibraryDescription))
				{
					if (_extensionDirectory == null)
					{
						string path = string.Format("{0}\\extensions",
							GridConstants.GridLibDirectory.FullName);
						_extensionDirectory = new DirectoryInfo(path);

						_extensionDirectory =
							GuaranteedDirectory.GetDirectory(
								_extensionDirectory);
					}
				}

				return _extensionDirectory;
			}
		}

		/// <summary>
		/// Retrieve the FileInfo for the file that represents this assembly.
		/// </summary>
		[XmlIgnore]
		public FileInfo AssemblyFile
		{
			get
			{
				return new FileInfo(string.Format("{0}/{1}",
					ExtensionDirectory.FullName, _filename));
			}
		}

		/// <summary>
		/// Overloaded the toString method to make pretty printing more ... pretty.
		/// </summary>
		/// <returns>The properly formatted string.</returns>
		public override string ToString()
		{
			if (_description != null)
				return string.Format("{0} -- {1}", _name, _description);
			else
				return _name;
		}

		/// <summary>
		/// Overloaded Equals so that we can remove it from collections.
		/// </summary>
		/// <param name="obj">The object to compare "this" one against.</param>
		/// <returns>true if they are equal, false otherwise.</returns>
		public override bool Equals(object obj)
		{
			ExtensionLibraryDescription other = (ExtensionLibraryDescription)obj;

			return _name.Equals(other._name);
		}

		/// <summary>
		/// Required to overload this if you overload equals.
		/// </summary>
		/// <returns>The hash code for this object instance.</returns>
		public override int GetHashCode()
		{
			return _name.GetHashCode();
		}

		/// <summary>
		/// Retrieve the file information for the configuration file.
		/// </summary>
		[XmlIgnore]
		static public FileInfo Configuration
		{
			get
			{
				return new FileInfo(
					string.Format("{0}/{1}", ExtensionDirectory.FullName,
						_CONFIG_FILE_NAME));
			}
		}

		/// <summary>
		/// Try and load the assembly from the given path.
		/// </summary>
		/// <param name="file">The file to load as an assembly.
		/// </param>
		/// <returns>The newly loaded assembly.</returns>
		static public Assembly loadAssembly(FileInfo file)
		{
			return Assembly.LoadFrom(file.FullName);
		}

		static private IEntryFactory[] loadEntryFactories(Assembly assembly)
		{
			ArrayList factories = new ArrayList();
			Type[] types = assembly.GetTypes();
			foreach (Type type in types)
			{
				if (typeof(IEntryFactory).IsAssignableFrom(type))
				{
					factories.Add(instantiateFactory(type));
				}
			}

			IEntryFactory []ret = new IEntryFactory[factories.Count];
			factories.CopyTo(ret);
			return ret;
		}

		static private ICommandFactory[] loadCommandFactories(Assembly assembly)
		{
			ArrayList factories = new ArrayList();
			Type[] types = assembly.GetTypes();
			foreach (Type type in types)
			{
				if (typeof(ICommandFactory).IsAssignableFrom(type))
				{
					factories.Add(instantiateFactory(type));
				}
			}

			ICommandFactory []ret = new ICommandFactory[factories.Count];
			factories.CopyTo(ret);
			return ret;
		}

		static private object instantiateFactory(Type type)
		{
			ConstructorInfo cons = type.GetConstructor(new Type[0]);
			return cons.Invoke(new object[0]);
		}
	}
}