// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;

namespace Grid.Operations
{
	/// <summary>
	/// Colelcts together all forms of Cp operation.
	/// </summary>
	public class GridCpOperation : IOperation
	{
		/// <summary>
		/// The Cp block size.
		/// </summary>
		private const int BUFFER_SIZE = 1024 * 8;

		#region Public Members

		/// <summary>
		/// Perform a cp operation with a collection of path expressions that
		/// may contain wildcards.  This form of Cp supports the notion of
		/// multiple sources and a single destination context.
		/// </summary>
		/// <param name="paths">The path expressions that make up our
		/// cp operation.</param>
		static public void cp(PathExpressionCollection paths)
		{
			IEntry []entries = paths.resolveAll();
			if (entries.Length < 2)
				throw new CpException("No target specified for cp.");

			if (entries.Length > 2)
				cp(entries);
			else
				cp(entries[0], entries[1]);
		}

		/// <summary>
		/// Copy a single source to a single destination.  Both of
		/// these strings must resolve to singletons.
		/// </summary>
		/// <param name="source">A path expression that must resolve to a
		/// single file.</param>
		/// <param name="target">A path expression that must resolve to a
		/// single context or file.</param>
		static public void cp(string source, string target)
		{
			cp(
				ContextUtilities.PathLookup(source,
					PathLookupFlags.MustExist),
				ContextUtilities.PathLookup(target));
		}

		/// <summary>
		/// Copy a single source to a single destination.  Both of
		/// these strings must resolve to singletons.
		/// </summary>
		/// <param name="source">An entry that represents a single file.
		/// </param>
		/// <param name="target">An entry that represents a single
		/// file or context.</param>
		static public void cp(IEntry source, IEntry target)
		{
			if (!source.Exists)
				throw new NoSuchPathException(source.Path);
 
			// We can't copy things that aren't files.
			if (!source.EntryType.Equals(WellknownEntryTypes.ENTRY_TYPE_FILE))
				throw new CpException(
					string.Format("Source path \"{0}\" is not a file.", source.Path));

			// If the target doesn't exist, then create it as a new file.
			if (!target.Exists)
			{
				target = ((INotAvailableEntry)target).createFile(false, null);
			}
				// Otherwise, if the target is a context, then make a new file with the
				// same name in that target context.
			else if (target.EntryType.Equals(WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
			{
				target = ((IContext)target).touch(source.EntryName, null);
			} 
				// Finally, if we aren't a file, then we can't copy to it.
			else if (!target.EntryType.Equals(WellknownEntryTypes.ENTRY_TYPE_FILE))
			{
				throw new CpException(
					string.Format("Target path \"{0}\" is not a file or context.",
						target.Path));
			}

			copySingle((IFile)source, (IFile)target);
		}

		/// <summary>
		/// Copy a source or collection of sources to a single target.
		/// </summary>
		/// <param name="paths">A collection of paths to make part of the copy
		/// command.  At least two must be given.  If more then two are given,
		/// then the final entry must be a context.</param>
		static public void cp(IEntry []paths)
		{
			IEntry targetDir = paths[paths.Length - 1];
			if (!targetDir.Exists)
				throw new NoSuchPathException(targetDir.Path);
			if (!targetDir.EntryType.Equals(WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
				throw new CpException(
					string.Format("Path \"{0}\" does not appear to be a context.",
						targetDir.Path));

			for (int lcv = 0; lcv < paths.Length - 1; lcv++)
				cp(paths[lcv], targetDir);
		}

		#endregion
		
		#region Private Members

		static private void copySingle(IFile source, IFile target)
		{
			int bytesRead;
			byte []buffer = new byte[BUFFER_SIZE];

			using (Stream reader = source.OpenRead())
			{
				using (Stream writer = target.Open(
						   FileMode.Create))
				{
					while ((bytesRead = reader.Read(
						buffer, 0, BUFFER_SIZE)) > 0)
					{
						writer.Write(buffer, 0, bytesRead);
					}
				}
			}
		}

		#endregion
	}
}