// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.WS.Addressing;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;

namespace Grid.Operations
{
	/// <summary>
	/// Collects together all operations that are part of an info command.  As
	/// one of the "display" operations, this operation currently does in fact
	/// display the output though in the future it will return some structure
	/// which represents the information to be displayed.
	/// </summary>
	public class GridInfoOperation : IOperation
	{
		/// <summary>
		/// Retrieve information on a bunch of entries.
		/// </summary>
		/// <param name="paths">The collection of paths to display information
		/// about.  All of the entries in this collection must represent grid
		/// entries and not local file system entries.</param>
		/// <param name="showEPR">Is this a showEPR operation.  So far, this
		/// is the only valid type of operation so it is assumed.  Later,
		/// we'll have other types of information to display.</param>
		static public void info(PathExpressionCollection paths, bool showEPR)
		{
			bool first = true;
			IEntry []entries = paths.resolveAll();
			foreach (IEntry entry in entries)
			{
				if (!first)
				{
					GridLibEnvironment.Instance.Console.Output.WriteLine();
					first = false;
				}

				printInfo(entry);
			}
		}

		/// <summary>
		/// Retrieve information on a bunch of entries.
		/// </summary>
		/// <param name="path">The path expression for entries to display 
		/// information about.  All resultant entries must represent grid
		/// entries and not local file system entries.</param>
		/// <param name="showEPR">Is this a showEPR operation.  So far, this
		/// is the only valid type of operation so it is assumed.  Later,
		/// we'll have other types of information to display.</param>
		static public void info(string path, bool showEPR)
		{
			info((IGridEntry)ContextUtilities.PathLookup(path,
				PathLookupFlags.MustExist |
				PathLookupFlags.Grid), showEPR);
		}

		/// <summary>
		/// Retrieve information on an entry.
		/// </summary>
		/// <param name="entry">The entry, which must represent grid
		/// entry and not local file system entry, that we want information
		/// about.</param>
		/// <param name="showEPR">Is this a showEPR operation.  So far, this
		/// is the only valid type of operation so it is assumed.  Later,
		/// we'll have other types of information to display.</param>
		static public void info(IGridEntry entry, bool showEPR)
		{
			printInfo(entry);
		}

		#region Private Members

		static private void printInfo(IEntry entry)
		{
			if (entry.Local)
				throw new CommandException(string.Format(
					"Can't determine EPR of local entry \"{0}\".",
					entry.Path));

			if (!entry.Exists)
				throw new NoSuchPathException(entry.Path);

			XmlRootAttribute root = new XmlRootAttribute(
				GridConstants.EPR_INFO_ROOT);
			root.Namespace = GridConstants.GRID_NS;

			XmlSerializer ser = new XmlSerializer(
				typeof(EndpointReferenceType), root);
			ser.Serialize(GridLibEnvironment.Instance.Console.Output,
				((IGridEntry)entry).EPR);
		}

		#endregion
	}
}