// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;
using Grid.Proxies;

using UVa.GCG.WSRF.Common.WS.Addressing;

namespace Grid.Operations
{
	/// <summary>
	/// Collects together all operations having to do with initializing
	/// a new grid.
	/// </summary>
	public class GridInitOperation
		: IOperation
	{
		/// <summary>
		/// Show a sample configuration file.
		/// </summary>
		static public void showExampleConfig()
		{
			showConfig();
		}

		/// <summary>
		/// Initialize a new grid.
		/// </summary>
		/// <param name="config">The configuration information for the
		/// new grid.</param>
		/// <param name="outputFile">An output file where context information
		/// (that can later be used to connect to the grid) is stored.</param>
		static public void init(GridInitConfig config, string outputFile)
		{
			GridLibEnvironment.Instance.Console.Trace.WriteLine(
				"Running grid init.");

			Hashtable bootstrapHints = getBootstrapHints(config);

			GridLibEnvironment.Instance.Console.Trace.WriteLine(
				"Creating root context.");

			CurrentContextEnvironment env = new CurrentContextEnvironment();
			// First, we create a new root context and store the environment.
			EndpointReferenceType root = 
				env.RootContext = env.CurrentContext = createRoot(config);
			CurrentContextEnvironment.store(env);

			// Next, we create . and .. entries for it.
			GridLibEnvironment.Instance.Console.Trace.WriteLine(
				"Setting up context.");
			GridContext c = new GridContext(root, "/", null);
			c.link(WellknownEntryTypes.ENTRY_TYPE_CONTEXT, ".", root, true);
			c.link(WellknownEntryTypes.ENTRY_TYPE_CONTEXT, "..", root, true);

			// Now we add the /System context
			GridLibEnvironment.Instance.Console.Trace.WriteLine(
				"Creating system directory.");
			GridContext system = (GridContext)c.createChild(
				WellknownEntryTypes.ENTRY_TYPE_CONTEXT, "System", bootstrapHints);

			// Finally, we create the /Factories subdir of /System and add all
			// the initial factory EPRs to it.
			GridLibEnvironment.Instance.Console.Trace.WriteLine(
				"Creating factories directory.");
			createFactories(config, system, bootstrapHints);

			// Right before we exit, we store the grid file that describes this
			// grid so others can connect to it.
			GridLibEnvironment.Instance.Console.Trace.WriteLine("Storing grid file.");
			CurrentContextEnvironment.store(
				GridLibEnvironment.Instance.ContextEnvironment, outputFile);
		}

		#region Private Members

		static private void createFactories(GridInitConfig config,
			GridContext c, Hashtable bootstrapHints)
		{
			GridContext factories = (GridContext)c.createChild(
				WellknownEntryTypes.ENTRY_TYPE_CONTEXT, "Factories",
				bootstrapHints);

			foreach (GridFactoryInfo info in config.Factories)
			{
				factories.link(WellknownEntryTypes.ENTRY_TYPE_FACTORY,
					info.FactoryName, info.FactoryEPR, true);
			}
		}

		static private void showConfig()
		{
			GridInitConfig conf = new GridInitConfig();
			conf.Factories = new GridFactoryInfo[]
				{
					new GridFactoryInfo(WellknownEntryTypes.ENTRY_TYPE_CONTEXT,
					"DefaultContextFactory",
					new EndpointReferenceType(
					new AttributedURI(null, 
					"http://localhost/GBG/BasicContexts/BasicContextService.asmx"), 
					null, null, null, null, null)),
					new GridFactoryInfo(WellknownEntryTypes.ENTRY_TYPE_FILE,
					"DefaultFileFactory",
					new EndpointReferenceType(
					new AttributedURI(null, 
					"http://localhost/GBG/BasicFiles/BasicFileService.asmx"), 
					null, null, null, null, null))
				};

			XmlSerializer ser = new XmlSerializer(
				typeof(GridInitConfig));
			ser.Serialize(GridLibEnvironment.Instance.Console.Output, conf);
		}

		static private EndpointReferenceType createRoot(GridInitConfig config)
		{
			FactoryServiceProxy proxy = new FactoryServiceProxy(
				findContextEPR(config));

			return proxy.create(null);
		}

		static private Hashtable getBootstrapHints(GridInitConfig config)
		{
			EndpointReferenceType epr = findContextEPR(config);
			Hashtable table = new Hashtable();

			table[WellknownCreationHints.CREATION_FACTORY] = epr;
			return table;
		}

		static private EndpointReferenceType findContextEPR(GridInitConfig config)
		{
			foreach (GridFactoryInfo info in config.Factories)
			{
				if (info.FactoryType.Equals(
					WellknownEntryTypes.ENTRY_TYPE_CONTEXT.ToString()))
					return info.FactoryEPR;
			}

			throw new GridInitException(
				"Grid Config file does not contain a declaration"
				+ " for a context factory.");
		}

		#endregion
	}
}