// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Xml;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;

namespace Grid.Operations
{
	/// <summary>
	///  Collects together all the ln operations available.
	/// </summary>
	public class GridLnOperation : IOperation
	{
		/// <summary>
		/// Link a source description to a target path.
		/// </summary>
		/// <param name="sourceDesc">A 
		/// <see cref="Grid.ContextSpace.GridEndpointDescription"/>
		/// GridEndpointDescription string that
		/// indicates what the source of this link is to be.</param>
		/// <param name="targetPath">A target path for this link.  This target
		/// path MUST resolve to a singleton or must be null.</param>
		/// <param name="suggestedType">The suggested XML type for the target.
		/// If none is given, an attempt might be made to figure it out, but
		/// is not guaranteed to work in which case the entry will be given
		/// an "unknown" type.</param>
		static public void ln(string sourceDesc,
			PathExpressionCollection targetPath, XmlQualifiedName suggestedType)
		{
			GridEndpointDescription gridSourceDesc
				= GridEndpointDescription.create(sourceDesc, suggestedType);

			IEntry source = gridSourceDesc.Entry;
			IEntry target;
			GridContext targetDir;
			string targetName;

			if (!source.Exists)
				throw new NoSuchPathException(source.Path);
			if (source.Local)
				throw new CommandException(
					"Not allowed to create a link from a local source.");

			if (targetPath == null)
				target = new GridContext(
					GridLibEnvironment.Instance.ContextEnvironment.CurrentContext, 
					".", null);
			else
				target = targetPath.resolveSingleton();

			if (target.Local)
				throw new CommandException(
					"Can't create grid links to local space.");

			if (target.Exists)
			{
				if (!target.EntryType.Equals(
					WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
						throw new PathAlreadyExistsException(target.Path);
				targetDir = (GridContext)target;
				targetName = source.EntryName;

				if (targetDir[targetName] != null)
					throw new PathAlreadyExistsException(
						string.Format("{0}/{1}", targetDir.Path, targetName));
			} 
			else
			{
				targetName = target.EntryName;
				target = target.Parent;

				if (!target.Exists)
					throw new NoSuchPathException(target.Path);
				if (!target.EntryType.Equals(
						WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
					throw new CommandException(
						string.Format("Path \"{0}\" is not a context.", target.Path));

				targetDir = (GridContext)target;
			}

			if (targetName == null)
				throw new CommandException(
					"Unable to determine target entry name.");

			ln((IGridEntry)source, targetDir, targetName);
		}

		#region Public Members

		/// <summary>
		/// Create a link from a source grid entry to a target context.
		/// </summary>
		/// <param name="source">The source grid entry to create the link
		/// from.</param>
		/// <param name="target">A target GridContext into which a new links
		/// is to be placed.</param>
		/// <param name="entryName">The entry name for the new link.  If this
		/// parameter is null, then the source's entry name will be used.
		/// </param>
		static public void ln(IGridEntry source, GridContext target,
			string entryName)
		{
			target.link(source.EntryType, entryName, source.EPR, true);
		}

		#endregion
	}
}