// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;

namespace Grid.Operations
{
	/// <summary>
	/// Collects together all of the valid ls operations.
	/// </summary>
	public class GridLsOperation : IOperation
	{
		/// <summary>
		/// List the contents of a given path expression.
		/// </summary>
		/// <param name="pathExpr">An expression that refers to a path or
		/// bunch of paths (wildcarded) that you wish to perform an ls of.
		/// </param>
		/// <param name="isLong">Should a long listing be performed?</param>
		/// <param name="all">Are we showing all files, or only the ones
		/// that aren't hidden.</param>
		/// <param name="nodir">When true, don't expand directories given as the
		/// path expression.  Otherwise, expand them.</param>
		static public void ls(string pathExpr, bool isLong, bool all, bool nodir)
		{
			ls(new PathExpressionCollection(pathExpr), isLong, all, nodir);
		}

		/// <summary>
		/// List the contents of the current directory.
		/// </summary>
		/// <param name="isLong">Should a long listing be performed?</param>
		/// <param name="all">Are we showing all files, or only the ones
		/// that aren't hidden.</param>
		/// <param name="nodir">When true, don't expand directories given as the
		/// path expression.  Otherwise, expand them.</param>
		static public void ls(bool isLong, bool all, bool nodir)
		{
			ls((PathExpressionCollection)null, isLong, all, nodir);
		}

		/// <summary>
		/// List the contents of a given path expression collection.
		/// </summary>
		/// <param name="paths">A collection of paths that you wish to 
		/// perform an ls of.</param>
		/// <param name="isLong">Should a long listing be performed?</param>
		/// <param name="all">Are we showing all files, or only the ones
		/// that aren't hidden.</param>
		/// <param name="nodir">When true, don't expand directories given as the
		/// path expression.  Otherwise, expand them.</param>
		static public void ls(PathExpressionCollection paths,
			bool isLong, bool all, bool nodir)
		{
			if (paths == null)
				paths = new PathExpressionCollection(".");

			bool first = true;
			IEntry []entries = paths.resolveAll();
			Array.Sort(entries, IEntryComparer.Instance);
			foreach (IEntry entry in entries)
			{
				displayEntry(entry, first, entries.Length > 1,
					isLong, all, nodir);
				first = false;
			}
		}

		static private void displayEntry(IEntry entry, bool first, bool showDir,
			bool isLong, bool all, bool nodir)
		{
			if (!entry.Exists)
				throw new NoSuchPathException(entry.Path);

			if (!nodir && entry.EntryType.Equals(
				WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
			{
				if (!first)
					GridLibEnvironment.Instance.Console.Output.WriteLine();

				if (showDir)
					GridLibEnvironment.Instance.Console.Output.WriteLine(
						"{0}:", entry.EntryName);

				IEntry []subEntries = ((IContext)entry).Children;
				Array.Sort(subEntries, IEntryComparer.Instance);
				foreach (IEntry subEntry in subEntries)
				{
					if (!all && subEntry.EntryName.StartsWith("."))
						continue;

					GridLibEnvironment.Instance.Console.Output.WriteLine(
						"{0}{1}",
						showDir ? "\t" : "",
						subEntry.format(isLong).ToString(isLong));
				}
			} 
			else
			{
				GridLibEnvironment.Instance.Console.Output.WriteLine(
					"{0}",
					entry.format(isLong).ToString(isLong));
			}
		}
	}
}