// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.Xml;
using System.Xml.Serialization;

using UVa.GCG.WSRF.Common.WS.Addressing;
using UVa.GCG.WSRF.Common.WS.ResourceProperties;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;

namespace Grid.Operations
{
	/// <summary>
	/// Collects together all methods which get properties of a grid entry.
	/// </summary>
	public class GridPropertiesOperation : IOperation
	{
		private const string _DEFAULT_QUERY = ".";

		/// <summary>
		/// Retrieve the ResourceProperties document for a given path.  This
		/// query assumes that the entire document is to be retrieved.
		/// </summary>
		/// <param name="path">The path to lookup.</param>
		/// <param name="isQuery">Is this a query?  So far, this is the only
		/// type of properties operation that can be performed so this should
		/// be true.</param>
		static public void properties(string path, bool isQuery)
		{
			properties(path, isQuery, _DEFAULT_QUERY);
		}

		/// <summary>
		/// Retrieve the ResourceProperties document for a given collection of
		/// paths.  This query assumes that the entire document is to be 
		/// retrieved.
		/// </summary>
		/// <param name="path">The collection of paths to lookup.</param>
		/// <param name="isQuery">Is this a query?  So far, this is the only
		/// type of properties operation that can be performed so this should
		/// be true.</param>
		static public void properties(PathExpressionCollection path,
			bool isQuery)
		{
			properties(path, isQuery, _DEFAULT_QUERY);
		}

		/// <summary>
		/// Retrieve the ResourceProperties document for a given entry.
		/// This query assumes that the entire document is to be 
		/// retrieved.
		/// </summary>
		/// <param name="entry">The grid entry to retrieve properties for.
		/// </param>
		/// <param name="isQuery">Is this a query?  So far, this is the only
		/// type of properties operation that can be performed so this should
		/// be true.</param>
		static public void properties(IGridEntry entry, bool isQuery)
		{
			properties(entry, isQuery, _DEFAULT_QUERY);
		}

		/// <summary>
		/// Retrieve the ResourceProperties document for a given path.  This
		/// query assumes that the entire document is to be retrieved.
		/// </summary>
		/// <param name="path">The path to lookup.</param>
		/// <param name="isQuery">Is this a query?  So far, this is the only
		/// type of properties operation that can be performed so this should
		/// be true.</param>
		/// <param name="query">The XPath Query to run over the 
		/// ResourceProperties document.</param>
		static public void properties(string path, bool isQuery, string query)
		{
			properties((IGridEntry)ContextUtilities.PathLookup(path,
				PathLookupFlags.MustExist |
				PathLookupFlags.Grid), isQuery, query);
		}

		/// <summary>
		/// Retrieve the ResourceProperties document for a given collection of
		/// paths.  This query assumes that the entire document is to be 
		/// retrieved.
		/// </summary>
		/// <param name="path">The collection of paths to lookup.</param>
		/// <param name="isQuery">Is this a query?  So far, this is the only
		/// type of properties operation that can be performed so this should
		/// be true.</param>
		/// <param name="query">The XPath Query to run over the 
		/// ResourceProperties document.</param>
		static public void properties(PathExpressionCollection path,
			bool isQuery, string query)
		{
			IEntry entry = path.resolveSingleton();
			if (entry.Local)
				throw new CommandException(
					"Can only query properties for grid entries.");

			properties((IGridEntry)entry, isQuery, query);
		}

		/// <summary>
		/// Retrieve the ResourceProperties document for a given entry.
		/// This query assumes that the entire document is to be 
		/// retrieved.
		/// </summary>
		/// <param name="entry">The grid entry to retrieve properties for.
		/// </param>
		/// <param name="isQuery">Is this a query?  So far, this is the only
		/// type of properties operation that can be performed so this should
		/// be true.</param>
		/// <param name="query">The XPath Query to run over the 
		/// ResourceProperties document.</param>
		static public void properties(IGridEntry entry, bool isQuery, string query)
		{
			XmlDocument doc = new XmlDocument();
			
			QueryResourcePropertiesBinding binding = 
				new QueryResourcePropertiesBinding(entry.EPR);
			QueryResourcePropertiesRequest request = 
				new QueryResourcePropertiesRequest();
			request.QueryExpression = new QueryExpressionType();
			request.QueryExpression.dialect = GridConstants.XPATH_DIALECT;
			request.QueryExpression.Any = new XmlNode[]
				{
					doc.CreateTextNode(query)
				};

			QueryResourcePropertiesResponse response =
				binding.QueryResourceProperties(request);

			bool first = true;
			foreach (XmlElement element in
				response.Any)
			{
				if (!first)
					GridLibEnvironment.Instance.Console.Output.WriteLine();
				first = false;

				GridLibEnvironment.Instance.Console.Output.WriteLine(
					element.OuterXml);
			}
		}
	}
}