// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;

using Grid.Commands;
using Grid.ContextSpace;
using Grid.Env;

namespace Grid.Operations
{
	/// <summary>
	/// Collects together all of the operations that perform rm operations.
	/// Currently this functionality isn't implement in grid lib.
	/// </summary>
	public class GridRmOperation : IOperation
	{
		/// <summary>
		/// Remove a given path from context space.
		/// </summary>
		/// <param name="path">A path or path expression to remove.</param>
		/// <param name="noDestroy">If true, then don't actually destroy the
		/// back end object.  Simply remove it's link from context space.  This
		/// parameter must be false if the target is a local path.
		/// </param>
		static public void rm(string path, bool noDestroy)
		{
			rm(new PathExpressionCollection(path), noDestroy);
		}

		/// <summary>
		/// Remove a given collection of paths from context space.
		/// </summary>
		/// <param name="paths">A collection of paths to remove.</param>
		/// <param name="noDestroy">If true, then don't actually destroy the
		/// back end object.  Simply remove it's link from context space.  This
		/// parameter must be false if the target is a local path.
		/// </param>
		static public void rm(PathExpressionCollection paths, bool noDestroy)
		{
			IEntry[] entries = paths.resolveAll();
			foreach (IEntry entry in entries)
			{
				if (!entry.Exists)
					throw new NoSuchPathException(entry.Path);

				if (entry.EntryType.Equals(
					WellknownEntryTypes.ENTRY_TYPE_CONTEXT))
					removeContext((IContext)entry, noDestroy);
				else
					removeEndpoint(entry, noDestroy);
			}
		}

		#region Public Methods

		/// <summary>
		/// Remove a given context from context space.
		/// </summary>
		/// <param name="context">The context to remove.</param>
		/// <param name="noDestroy">Whether or not to actually destroy
		/// the backend object.  For local directories, this must be
		/// false.</param>
		static public void removeContext(IContext context, bool noDestroy)
		{
			if (context.Local && noDestroy)
				throw new CommandException(
					"Unable to remove a local directory without destroying it.");

			throw new UnimplementedException(
				typeof(GridRmOperation), 
				typeof(GridRmOperation).GetMethod("removeContext"));
		}

		/// <summary>
		/// Remove a given entry from context space.
		/// </summary>
		/// <param name="entry">The entry to remove.</param>
		/// <param name="noDestroy">Whether or not to actually destroy
		/// the backend object.  For local directories, this must be
		/// false.</param>
		static public void removeEndpoint(IEntry entry, bool noDestroy)
		{
			if (entry.Local && noDestroy)
				throw new CommandException(
					"Unable to remove a local file without destroying it.");

			throw new UnimplementedException(typeof(GridRmOperation), 
				typeof(GridRmOperation).GetMethod("removeEndpoint"));
		}

		#endregion
	}
}