// Copyright (c) 2004, GBG, University of Virginia, All Rights Reserved
// Author: Mark Morgan, mmm2a@cs.virginia.edu

using System;
using System.IO;

namespace Grid.Output
{
	/// <summary>
	/// The OutputConsole is a collection of TextWriters that conveniently
	/// give a single point through with ALL console type IO should happen.
	/// This class allows us to reset the various consoles so that we can
	/// filter out which IO we do or don't want.
	/// </summary>
	public class OutputConsole
	{
		private TextWriter _trace = TextWriter.Null;
		private TextWriter _debug = TextWriter.Null;
		private TextWriter _output = Console.Out;
		private TextWriter _error = Console.Error;

		/// <summary>
		/// Construct a new OutputConsole.  Provides reasonable defaults for
		/// output, error, trace, and debug.
		/// </summary>
		public OutputConsole()
		{
		}

		/// <summary>
		/// Construct a new output console.
		/// </summary>
		/// <param name="output">The normal output stream.</param>
		/// <param name="error">The stream to which errors should
		/// be given.</param>
		/// <param name="trace">The stream to which trace information
		/// should be given.</param>
		/// <param name="debug">The stream to which debug information
		/// should be given.</param>
		public OutputConsole(TextWriter output, TextWriter error,
			TextWriter trace, TextWriter debug)
		{
			_output = output;
			_error = error;
			_trace = trace;
			_debug = debug;
		}

		/// <summary>
		/// Construct a new output console.  Sets the trace and debug streams
		/// to /dev/null.
		/// </summary>
		/// <param name="output">The stream to which output should go.</param>
		/// <param name="error">The stream to which error should go.</param>
		public OutputConsole(TextWriter output, TextWriter error)
			: this(output, error, TextWriter.Null, TextWriter.Null)
		{
		}

		/// <summary>
		/// Construct a new output console which is default except for where
		/// the output should go.
		/// </summary>
		/// <param name="output">An output stream to use.</param>
		public OutputConsole(TextWriter output)
			: this(output, TextWriter.Null)
		{
		}

		/// <summary>Get or set the Trace stream.</summary>
		public TextWriter Trace
		{
			get
			{
				return _trace;
			}

			set
			{
				_trace = value;
			}
		}

		/// <summary>Get or set the Debug stream.</summary>
		public TextWriter Debug
		{
			get
			{
				return _debug;
			}

			set
			{
				_debug = value;
			}
		}

		/// <summary>Get or set the Error stream.</summary>
		public TextWriter Error
		{
			get
			{
				return _error;
			}

			set
			{
				_error = value;
			}
		}

		/// <summary>Get or set the Output stream.</summary>
		public TextWriter Output
		{
			get
			{
				return _output;
			}

			set
			{
				_output = value;
			}
		}
	}
}